@@ -6,6 +6,7 @@ use serde::Serialize;
 pub fn cli() -> App {
     subcommand("locate-project")
         .about("Print a JSON representation of a Cargo.toml file's location")
+        .arg(opt("quiet", "No output printed to stdout").short("q"))
         .arg_manifest_path()
 }
 
@@ -20,7 +21,7 @@ pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
     let root = root
         .to_str()
         .ok_or_else(|| {
-            failure::format_err!(
+            anyhow::format_err!(
                 "your package path contains characters \
                  not representable in Unicode"
             )
