@@ -1,22 +1,23 @@
 use std::cell::RefCell;
 use std::collections::hash_map::{Entry, HashMap};
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashSet};
 use std::path::{Path, PathBuf};
 use std::slice;
 
 use glob::glob;
+use log::debug;
 use url::Url;
 
-use core::profiles::Profiles;
-use core::registry::PackageRegistry;
-use core::{Dependency, PackageIdSpec};
-use core::{EitherManifest, Package, SourceId, VirtualManifest};
-use ops;
-use sources::PathSource;
-use util::errors::{CargoResult, CargoResultExt, ManifestError};
-use util::paths;
-use util::toml::read_manifest;
-use util::{Config, Filesystem};
+use crate::core::features::Features;
+use crate::core::registry::PackageRegistry;
+use crate::core::{Dependency, PackageId, PackageIdSpec};
+use crate::core::{EitherManifest, Package, SourceId, VirtualManifest};
+use crate::ops;
+use crate::sources::PathSource;
+use crate::util::errors::{CargoResult, CargoResultExt, ManifestError};
+use crate::util::paths;
+use crate::util::toml::{read_manifest, TomlProfiles};
+use crate::util::{Config, Filesystem};
 
 /// The core abstraction in Cargo for working with a workspace of crates.
 ///
@@ -50,24 +51,25 @@ pub struct Workspace<'cfg> {
     // paths. The packages themselves can be looked up through the `packages`
     // set above.
     members: Vec<PathBuf>,
+    member_ids: HashSet<PackageId>,
 
     // The subset of `members` that are used by the
     // `build`, `check`, `test`, and `bench` subcommands
-    // when no package is selected with `--package` / `-p` and `--all`
+    // when no package is selected with `--package` / `-p` and `--workspace`
     // is not used.
     //
     // This is set by the `default-members` config
     // in the `[workspace]` section.
     // When unset, this is the same as `members` for virtual workspaces
-    // (`--all` is implied)
+    // (`--workspace` is implied)
     // or only the root package for non-virtual workspaces.
     default_members: Vec<PathBuf>,
 
-    // True, if this is a temporary workspace created for the purposes of
-    // cargo install or cargo package.
+    // `true` if this is a temporary workspace created for the purposes of the
+    // `cargo install` or `cargo package` commands.
     is_ephemeral: bool,
 
-    // True if this workspace should enforce optional dependencies even when
+    // `true` if this workspace should enforce optional dependencies even when
     // not needed; false if this workspace should only enforce dependencies
     // needed by the current configuration (such as in cargo install). In some
     // cases `false` also results in the non-enforcement of dev-dependencies.
@@ -76,6 +78,10 @@ pub struct Workspace<'cfg> {
     // A cache of loaded packages for particular paths which is disjoint from
     // `packages` up above, used in the `load` method down below.
     loaded_packages: RefCell<HashMap<PathBuf, Package>>,
+
+    // If `true`, then the resolver will ignore any existing `Cargo.lock`
+    // file. This is set for `cargo install` without `--locked`.
+    ignore_lock: bool,
 }
 
 // Separate structure for tracking loaded packages (to avoid loading anything
@@ -118,7 +124,7 @@ pub struct WorkspaceRootConfig {
 
 /// An iterator over the member packages of a workspace, returned by
 /// `Workspace::members`
-pub struct Members<'a, 'cfg: 'a> {
+pub struct Members<'a, 'cfg> {
     ws: &'a Workspace<'cfg>,
     iter: slice::Iter<'a, PathBuf>,
 }
@@ -131,26 +137,49 @@ impl<'cfg> Workspace<'cfg> {
     /// root and all member packages. It will then validate the workspace
     /// before returning it, so `Ok` is only returned for valid workspaces.
     pub fn new(manifest_path: &Path, config: &'cfg Config) -> CargoResult<Workspace<'cfg>> {
-        let target_dir = config.target_dir()?;
+        let mut ws = Workspace::new_default(manifest_path.to_path_buf(), config);
+        ws.target_dir = config.target_dir()?;
+        ws.root_manifest = ws.find_root(manifest_path)?;
+        ws.find_members()?;
+        ws.validate()?;
+        Ok(ws)
+    }
 
-        let mut ws = Workspace {
+    fn new_default(current_manifest: PathBuf, config: &'cfg Config) -> Workspace<'cfg> {
+        Workspace {
             config,
-            current_manifest: manifest_path.to_path_buf(),
+            current_manifest,
             packages: Packages {
                 config,
                 packages: HashMap::new(),
             },
             root_manifest: None,
-            target_dir,
+            target_dir: None,
             members: Vec::new(),
+            member_ids: HashSet::new(),
             default_members: Vec::new(),
             is_ephemeral: false,
             require_optional_deps: true,
             loaded_packages: RefCell::new(HashMap::new()),
-        };
-        ws.root_manifest = ws.find_root(manifest_path)?;
+            ignore_lock: false,
+        }
+    }
+
+    pub fn new_virtual(
+        root_path: PathBuf,
+        current_manifest: PathBuf,
+        manifest: VirtualManifest,
+        config: &'cfg Config,
+    ) -> CargoResult<Workspace<'cfg>> {
+        let mut ws = Workspace::new_default(current_manifest, config);
+        ws.root_manifest = Some(root_path.join("Cargo.toml"));
+        ws.target_dir = config.target_dir()?;
+        ws.packages
+            .packages
+            .insert(root_path, MaybePackage::Virtual(manifest));
         ws.find_members()?;
-        ws.validate()?;
+        // TODO: validation does not work because it walks up the directory
+        // tree looking for the root which is a fake file that doesn't exist.
         Ok(ws)
     }
 
@@ -169,33 +198,21 @@ impl<'cfg> Workspace<'cfg> {
         target_dir: Option<Filesystem>,
         require_optional_deps: bool,
     ) -> CargoResult<Workspace<'cfg>> {
-        let mut ws = Workspace {
-            config,
-            current_manifest: package.manifest_path().to_path_buf(),
-            packages: Packages {
-                config,
-                packages: HashMap::new(),
-            },
-            root_manifest: None,
-            target_dir: None,
-            members: Vec::new(),
-            default_members: Vec::new(),
-            is_ephemeral: true,
-            require_optional_deps,
-            loaded_packages: RefCell::new(HashMap::new()),
+        let mut ws = Workspace::new_default(package.manifest_path().to_path_buf(), config);
+        ws.is_ephemeral = true;
+        ws.require_optional_deps = require_optional_deps;
+        let key = ws.current_manifest.parent().unwrap();
+        let id = package.package_id();
+        let package = MaybePackage::Package(package);
+        ws.packages.packages.insert(key.to_path_buf(), package);
+        ws.target_dir = if let Some(dir) = target_dir {
+            Some(dir)
+        } else {
+            ws.config.target_dir()?
         };
-        {
-            let key = ws.current_manifest.parent().unwrap();
-            let package = MaybePackage::Package(package);
-            ws.packages.packages.insert(key.to_path_buf(), package);
-            ws.target_dir = if let Some(dir) = target_dir {
-                Some(dir)
-            } else {
-                ws.config.target_dir()?
-            };
-            ws.members.push(ws.current_manifest.clone());
-            ws.default_members.push(ws.current_manifest.clone());
-        }
+        ws.members.push(ws.current_manifest.clone());
+        ws.member_ids.insert(id);
+        ws.default_members.push(ws.current_manifest.clone());
         Ok(ws)
     }
 
@@ -206,7 +223,7 @@ impl<'cfg> Workspace<'cfg> {
     /// indicating that something else should be passed.
     pub fn current(&self) -> CargoResult<&Package> {
         let pkg = self.current_opt().ok_or_else(|| {
-            format_err!(
+            anyhow::format_err!(
                 "manifest path `{}` is a virtual manifest, but this \
                  command requires running against an actual package in \
                  this workspace",
@@ -216,6 +233,19 @@ impl<'cfg> Workspace<'cfg> {
         Ok(pkg)
     }
 
+    pub fn current_mut(&mut self) -> CargoResult<&mut Package> {
+        let cm = self.current_manifest.clone();
+        let pkg = self.current_opt_mut().ok_or_else(|| {
+            anyhow::format_err!(
+                "manifest path `{}` is a virtual manifest, but this \
+                 command requires running against an actual package in \
+                 this workspace",
+                cm.display()
+            )
+        })?;
+        Ok(pkg)
+    }
+
     pub fn current_opt(&self) -> Option<&Package> {
         match *self.packages.get(&self.current_manifest) {
             MaybePackage::Package(ref p) => Some(p),
@@ -223,6 +253,13 @@ impl<'cfg> Workspace<'cfg> {
         }
     }
 
+    pub fn current_opt_mut(&mut self) -> Option<&mut Package> {
+        match *self.packages.get_mut(&self.current_manifest) {
+            MaybePackage::Package(ref mut p) => Some(p),
+            MaybePackage::Virtual(..) => None,
+        }
+    }
+
     pub fn is_virtual(&self) -> bool {
         match *self.packages.get(&self.current_manifest) {
             MaybePackage::Package(..) => false,
@@ -235,13 +272,10 @@ impl<'cfg> Workspace<'cfg> {
         self.config
     }
 
-    pub fn profiles(&self) -> &Profiles {
-        let root = self.root_manifest
-            .as_ref()
-            .unwrap_or(&self.current_manifest);
-        match *self.packages.get(root) {
-            MaybePackage::Package(ref p) => p.manifest().profiles(),
-            MaybePackage::Virtual(ref vm) => vm.profiles(),
+    pub fn profiles(&self) -> Option<&TomlProfiles> {
+        match self.root_maybe() {
+            MaybePackage::Package(p) => p.manifest().profiles(),
+            MaybePackage::Virtual(vm) => vm.profiles(),
         }
     }
 
@@ -253,8 +287,18 @@ impl<'cfg> Workspace<'cfg> {
         match self.root_manifest {
             Some(ref p) => p,
             None => &self.current_manifest,
-        }.parent()
-            .unwrap()
+        }
+        .parent()
+        .unwrap()
+    }
+
+    /// Returns the root Package or VirtualManifest.
+    fn root_maybe(&self) -> &MaybePackage {
+        let root = self
+            .root_manifest
+            .as_ref()
+            .unwrap_or(&self.current_manifest);
+        self.packages.get(root)
     }
 
     pub fn target_dir(&self) -> Filesystem {
@@ -263,31 +307,23 @@ impl<'cfg> Workspace<'cfg> {
             .unwrap_or_else(|| Filesystem::new(self.root().join("target")))
     }
 
-    /// Returns the root [replace] section of this workspace.
+    /// Returns the root `[replace]` section of this workspace.
     ///
     /// This may be from a virtual crate or an actual crate.
     pub fn root_replace(&self) -> &[(PackageIdSpec, Dependency)] {
-        let path = match self.root_manifest {
-            Some(ref p) => p,
-            None => &self.current_manifest,
-        };
-        match *self.packages.get(path) {
-            MaybePackage::Package(ref p) => p.manifest().replace(),
-            MaybePackage::Virtual(ref vm) => vm.replace(),
+        match self.root_maybe() {
+            MaybePackage::Package(p) => p.manifest().replace(),
+            MaybePackage::Virtual(vm) => vm.replace(),
         }
     }
 
-    /// Returns the root [patch] section of this workspace.
+    /// Returns the root `[patch]` section of this workspace.
     ///
     /// This may be from a virtual crate or an actual crate.
     pub fn root_patch(&self) -> &HashMap<Url, Vec<Dependency>> {
-        let path = match self.root_manifest {
-            Some(ref p) => p,
-            None => &self.current_manifest,
-        };
-        match *self.packages.get(path) {
-            MaybePackage::Package(ref p) => p.manifest().patch(),
-            MaybePackage::Virtual(ref vm) => vm.patch(),
+        match self.root_maybe() {
+            MaybePackage::Package(p) => p.manifest().patch(),
+            MaybePackage::Virtual(vm) => vm.patch(),
         }
     }
 
@@ -309,7 +345,7 @@ impl<'cfg> Workspace<'cfg> {
 
     /// Returns true if the package is a member of the workspace.
     pub fn is_member(&self, pkg: &Package) -> bool {
-        self.members().any(|p| p == pkg)
+        self.member_ids.contains(&pkg.package_id())
     }
 
     pub fn is_ephemeral(&self) -> bool {
@@ -320,14 +356,23 @@ impl<'cfg> Workspace<'cfg> {
         self.require_optional_deps
     }
 
-    pub fn set_require_optional_deps<'a>(
-        &'a mut self,
+    pub fn set_require_optional_deps(
+        &mut self,
         require_optional_deps: bool,
     ) -> &mut Workspace<'cfg> {
         self.require_optional_deps = require_optional_deps;
         self
     }
 
+    pub fn ignore_lock(&self) -> bool {
+        self.ignore_lock
+    }
+
+    pub fn set_ignore_lock(&mut self, ignore_lock: bool) -> &mut Workspace<'cfg> {
+        self.ignore_lock = ignore_lock;
+        self
+    }
+
     /// Finds the root of a workspace for the crate whose manifest is located
     /// at `manifest_path`.
     ///
@@ -415,6 +460,10 @@ impl<'cfg> Workspace<'cfg> {
                 debug!("find_members - only me as a member");
                 self.members.push(self.current_manifest.clone());
                 self.default_members.push(self.current_manifest.clone());
+                if let Ok(pkg) = self.current() {
+                    let id = pkg.package_id();
+                    self.member_ids.insert(id);
+                }
                 return Ok(());
             }
         };
@@ -425,15 +474,19 @@ impl<'cfg> Workspace<'cfg> {
             let root_package = self.packages.load(&root_manifest_path)?;
             match *root_package.workspace_config() {
                 WorkspaceConfig::Root(ref root_config) => {
-                    members_paths =
-                        root_config.members_paths(root_config.members.as_ref().unwrap_or(&vec![]))?;
-                    default_members_paths = if let Some(ref default) = root_config.default_members {
-                        Some(root_config.members_paths(default)?)
+                    members_paths = root_config
+                        .members_paths(root_config.members.as_ref().unwrap_or(&vec![]))?;
+                    default_members_paths = if root_manifest_path == self.current_manifest {
+                        if let Some(ref default) = root_config.default_members {
+                            Some(root_config.members_paths(default)?)
+                        } else {
+                            None
+                        }
                     } else {
                         None
-                    }
+                    };
                 }
-                _ => bail!(
+                _ => anyhow::bail!(
                     "root of a workspace inferred but wasn't a root: {}",
                     root_manifest_path.display()
                 ),
@@ -448,7 +501,7 @@ impl<'cfg> Workspace<'cfg> {
             for path in default {
                 let manifest_path = paths::normalize_path(&path.join("Cargo.toml"));
                 if !self.members.contains(&manifest_path) {
-                    bail!(
+                    anyhow::bail!(
                         "package `{}` is listed in workspace’s default-members \
                          but is not a member.",
                         path.display()
@@ -475,7 +528,8 @@ impl<'cfg> Workspace<'cfg> {
         if self.members.contains(&manifest_path) {
             return Ok(());
         }
-        if is_path_dep && !manifest_path.parent().unwrap().starts_with(self.root())
+        if is_path_dep
+            && !manifest_path.parent().unwrap().starts_with(self.root())
             && self.find_root(&manifest_path)? != self.root_manifest
         {
             // If `manifest_path` is a path dependency outside of the workspace,
@@ -499,6 +553,7 @@ impl<'cfg> Workspace<'cfg> {
                 MaybePackage::Package(ref p) => p,
                 MaybePackage::Virtual(_) => return Ok(()),
             };
+            self.member_ids.insert(pkg.package_id());
             pkg.dependencies()
                 .iter()
                 .map(|d| d.source_id())
@@ -514,12 +569,20 @@ impl<'cfg> Workspace<'cfg> {
         Ok(())
     }
 
+    pub fn features(&self) -> &Features {
+        match self.root_maybe() {
+            MaybePackage::Package(p) => p.manifest().features(),
+            MaybePackage::Virtual(vm) => vm.features(),
+        }
+    }
+
     /// Validates a workspace, ensuring that a number of invariants are upheld:
     ///
     /// 1. A workspace only has one root.
     /// 2. All workspace members agree on this one root as the root.
     /// 3. The current crate is a member of this workspace.
     fn validate(&mut self) -> CargoResult<()> {
+        // The rest of the checks require a VirtualManifest or multiple members.
         if self.root_manifest.is_none() {
             return Ok(());
         }
@@ -540,7 +603,7 @@ impl<'cfg> Workspace<'cfg> {
                     MaybePackage::Virtual(_) => continue,
                 };
                 if let Some(prev) = names.insert(name, member) {
-                    bail!(
+                    anyhow::bail!(
                         "two packages named `{}` in this workspace:\n\
                          - {}\n\
                          - {}",
@@ -553,7 +616,7 @@ impl<'cfg> Workspace<'cfg> {
         }
 
         match roots.len() {
-            0 => bail!(
+            0 => anyhow::bail!(
                 "`package.workspace` configuration points to a crate \
                  which is not configured with [workspace]: \n\
                  configuration at: {}\n\
@@ -563,7 +626,7 @@ impl<'cfg> Workspace<'cfg> {
             ),
             1 => {}
             _ => {
-                bail!(
+                anyhow::bail!(
                     "multiple workspace roots found in the same workspace:\n{}",
                     roots
                         .iter()
@@ -582,7 +645,7 @@ impl<'cfg> Workspace<'cfg> {
 
             match root {
                 Some(root) => {
-                    bail!(
+                    anyhow::bail!(
                         "package `{}` is a member of the wrong workspace\n\
                          expected: {}\n\
                          actual:   {}",
@@ -592,7 +655,7 @@ impl<'cfg> Workspace<'cfg> {
                     );
                 }
                 None => {
-                    bail!(
+                    anyhow::bail!(
                         "workspace member `{}` is not hierarchically below \
                          the workspace root `{}`",
                         member.display(),
@@ -644,10 +707,13 @@ impl<'cfg> Workspace<'cfg> {
                     }
                 }
             };
-            bail!(
+            anyhow::bail!(
                 "current package believes it's in a workspace when it's not:\n\
                  current:   {}\n\
-                 workspace: {}\n\n{}",
+                 workspace: {}\n\n{}\n\
+                 Alternatively, to keep it out of the workspace, add the package \
+                 to the `workspace.exclude` array, or add an empty `[workspace]` \
+                 table to the package's manifest.",
                 self.current_manifest.display(),
                 root.display(),
                 extra
@@ -655,21 +721,32 @@ impl<'cfg> Workspace<'cfg> {
         }
 
         if let Some(ref root_manifest) = self.root_manifest {
-            for pkg in self.members()
+            for pkg in self
+                .members()
                 .filter(|p| p.manifest_path() != root_manifest)
             {
-                if pkg.manifest().original().has_profiles() {
-                    let message = &format!(
-                        "profiles for the non root package will be ignored, \
-                         specify profiles at the workspace root:\n\
+                let manifest = pkg.manifest();
+                let emit_warning = |what| -> CargoResult<()> {
+                    let msg = format!(
+                        "{} for the non root package will be ignored, \
+                         specify {} at the workspace root:\n\
                          package:   {}\n\
                          workspace: {}",
+                        what,
+                        what,
                         pkg.manifest_path().display(),
-                        root_manifest.display()
+                        root_manifest.display(),
                     );
-
-                    //TODO: remove `Eq` bound from `Profiles` when the warning is removed.
-                    self.config.shell().warn(&message)?;
+                    self.config.shell().warn(&msg)
+                };
+                if manifest.original().has_profiles() {
+                    emit_warning("profiles")?;
+                }
+                if !manifest.replace().is_empty() {
+                    emit_warning("replace")?;
+                }
+                if !manifest.patch().is_empty() {
+                    emit_warning("patch")?;
                 }
             }
         }
@@ -680,7 +757,7 @@ impl<'cfg> Workspace<'cfg> {
     pub fn load(&self, manifest_path: &Path) -> CargoResult<Package> {
         match self.packages.maybe_get(manifest_path) {
             Some(&MaybePackage::Package(ref p)) => return Ok(p.clone()),
-            Some(&MaybePackage::Virtual(_)) => bail!("cannot load workspace root"),
+            Some(&MaybePackage::Virtual(_)) => anyhow::bail!("cannot load workspace root"),
             None => {}
         }
 
@@ -689,7 +766,7 @@ impl<'cfg> Workspace<'cfg> {
             return Ok(p);
         }
         let source_id = SourceId::for_path(manifest_path.parent().unwrap())?;
-        let (package, _nested_paths) = ops::read_package(manifest_path, &source_id, self.config)?;
+        let (package, _nested_paths) = ops::read_package(manifest_path, source_id, self.config)?;
         loaded.insert(manifest_path.to_path_buf(), package.clone());
         Ok(package)
     }
@@ -731,16 +808,22 @@ impl<'cfg> Workspace<'cfg> {
                 MaybePackage::Package(pkg) => pkg.manifest().warnings().warnings(),
                 MaybePackage::Virtual(vm) => vm.warnings().warnings(),
             };
+            let path = path.join("Cargo.toml");
             for warning in warnings {
                 if warning.is_critical {
-                    let err = format_err!("{}", warning.message);
-                    let cx = format_err!(
-                        "failed to parse manifest at `{}`",
-                        path.display()
-                    );
+                    let err = anyhow::format_err!("{}", warning.message);
+                    let cx =
+                        anyhow::format_err!("failed to parse manifest at `{}`", path.display());
                     return Err(err.context(cx).into());
                 } else {
-                    self.config.shell().warn(&warning.message)?
+                    let msg = if self.root_manifest.is_none() {
+                        warning.message.to_string()
+                    } else {
+                        // In a workspace, it can be confusing where a warning
+                        // originated, so include the path.
+                        format!("{}: {}", path.display(), warning.message)
+                    };
+                    self.config.shell().warn(msg)?
                 }
             }
         }
@@ -753,10 +836,18 @@ impl<'cfg> Packages<'cfg> {
         self.maybe_get(manifest_path).unwrap()
     }
 
+    fn get_mut(&mut self, manifest_path: &Path) -> &mut MaybePackage {
+        self.maybe_get_mut(manifest_path).unwrap()
+    }
+
     fn maybe_get(&self, manifest_path: &Path) -> Option<&MaybePackage> {
         self.packages.get(manifest_path.parent().unwrap())
     }
 
+    fn maybe_get_mut(&mut self, manifest_path: &Path) -> Option<&mut MaybePackage> {
+        self.packages.get_mut(manifest_path.parent().unwrap())
+    }
+
     fn load(&mut self, manifest_path: &Path) -> CargoResult<&MaybePackage> {
         let key = manifest_path.parent().unwrap();
         match self.packages.entry(key.to_path_buf()) {
@@ -764,7 +855,7 @@ impl<'cfg> Packages<'cfg> {
             Entry::Vacant(v) => {
                 let source_id = SourceId::for_path(key)?;
                 let (manifest, _nested_paths) =
-                    read_manifest(manifest_path, &source_id, self.config)?;
+                    read_manifest(manifest_path, source_id, self.config)?;
                 Ok(v.insert(match manifest {
                     EitherManifest::Real(manifest) => {
                         MaybePackage::Package(Package::new(manifest, manifest_path))
@@ -806,7 +897,7 @@ impl MaybePackage {
 }
 
 impl WorkspaceRootConfig {
-    /// Create a new Intermediate Workspace Root configuration.
+    /// Creates a new Intermediate Workspace Root configuration.
     pub fn new(
         root_dir: &Path,
         members: &Option<Vec<String>>,
@@ -823,9 +914,10 @@ impl WorkspaceRootConfig {
 
     /// Checks the path against the `excluded` list.
     ///
-    /// This method does NOT consider the `members` list.
+    /// This method does **not** consider the `members` list.
     fn is_excluded(&self, manifest_path: &Path) -> bool {
-        let excluded = self.exclude
+        let excluded = self
+            .exclude
             .iter()
             .any(|ex| manifest_path.starts_with(self.root_dir.join(ex)));
 
@@ -867,10 +959,13 @@ impl WorkspaceRootConfig {
             Some(p) => p,
             None => return Ok(Vec::new()),
         };
-        let res = glob(path).chain_err(|| format_err!("could not parse pattern `{}`", &path))?;
-        let res = res.map(|p| {
-            p.chain_err(|| format_err!("unable to match path to pattern `{}`", &path))
-        }).collect::<Result<Vec<_>, _>>()?;
+        let res =
+            glob(path).chain_err(|| anyhow::format_err!("could not parse pattern `{}`", &path))?;
+        let res = res
+            .map(|p| {
+                p.chain_err(|| anyhow::format_err!("unable to match path to pattern `{}`", &path))
+            })
+            .collect::<Result<Vec<_>, _>>()?;
         Ok(res)
     }
 }
