@@ -1,41 +1,46 @@
-//! Cargo `compile` currently does the following steps.
+//! The Cargo "compile" operation.
 //!
-//! All configurations are already injected as environment variables via the
-//! main cargo command.
+//! This module contains the entry point for starting the compilation process
+//! for commands like `build`, `test`, `doc`, `rustc`, etc.
 //!
-//! 1. Read the manifest.
-//! 2. Shell out to `cargo-resolve` with a list of dependencies and sources as
-//!    stdin.
+//! The `compile` function will do all the work to compile a workspace. A
+//! rough outline is:
 //!
-//!    a. Shell out to `--do update` and `--do list` for each source.
-//!    b. Resolve dependencies and return a list of name/version/source.
-//!
-//! 3. Shell out to `--do download` for each source.
-//! 4. Shell out to `--do get` for each source, and build up the list of paths
-//!    to pass to `rustc -L`.
-//! 5. Call `cargo-rustc` with the results of the resolver zipped together with
-//!    the results of the `get`.
-//!
-//!    a. Topologically sort the dependencies.
-//!    b. Compile each dependency in order, passing in the -L's pointing at each
-//!       previously compiled dependency.
+//! - Resolve the dependency graph (see `ops::resolve`).
+//! - Download any packages needed (see `PackageSet`). Note that dependency
+//!   downloads are deferred until `build_unit_dependencies`.
+//! - Generate a list of top-level "units" of work for the targets the user
+//!   requested on the command-line. Each `Unit` corresponds to a compiler
+//!   invocation. This is done in this module (`generate_targets`).
+//! - Create a `Context` which will perform the following steps:
+//!     - Build the graph of `Unit` dependencies (see
+//!       `core::compiler::context::unit_dependencies`).
+//!     - Prepare the `target` directory (see `Layout`).
+//!     - Create a job queue (see `JobQueue`). The queue checks the
+//!       fingerprint of each `Unit` to determine if it should run or be
+//!       skipped.
+//!     - Execute the queue. Each leaf in the queue's dependency graph is
+//!       executed, and then removed from the graph when finished. This
+//!       repeats until the queue is empty.
 
 use std::collections::{BTreeSet, HashMap, HashSet};
 use std::iter::FromIterator;
 use std::path::PathBuf;
 use std::sync::Arc;
 
-use crate::core::compiler::{
-    BuildConfig, BuildContext, Compilation, Context, DefaultExecutor, Executor,
-};
-use crate::core::compiler::{CompileMode, Kind, Unit};
+use crate::core::compiler::standard_lib;
+use crate::core::compiler::unit_dependencies::build_unit_dependencies;
+use crate::core::compiler::{BuildConfig, BuildContext, Compilation, Context};
+use crate::core::compiler::{CompileKind, CompileMode, Unit};
+use crate::core::compiler::{DefaultExecutor, Executor, UnitInterner};
 use crate::core::profiles::{Profiles, UnitFor};
-use crate::core::resolver::{Method, Resolve};
-use crate::core::{Package, Source, Target};
+use crate::core::resolver::{Resolve, ResolveOpts};
+use crate::core::{LibKind, Package, PackageSet, Target};
 use crate::core::{PackageId, PackageIdSpec, TargetKind, Workspace};
 use crate::ops;
+use crate::ops::resolve::WorkspaceResolve;
 use crate::util::config::Config;
-use crate::util::{lev_distance, profile, CargoResult};
+use crate::util::{closest_msg, profile, CargoResult};
 
 /// Contains information about how a package should be compiled.
 #[derive(Debug)]
@@ -61,6 +66,9 @@ pub struct CompileOptions<'a> {
     pub target_rustc_args: Option<Vec<String>>,
     /// Extra arguments passed to all selected targets for rustdoc.
     pub local_rustdoc_args: Option<Vec<String>>,
+    /// Whether the `--document-private-items` flags was specified and should
+    /// be forwarded to `rustdoc`.
+    pub rustdoc_document_private_items: bool,
     /// The directory to copy final artifacts to. Note that even if `out_dir` is
     /// set, a copy of artifacts still could be found a `target/(debug\release)`
     /// as usual.
@@ -84,6 +92,7 @@ impl<'a> CompileOptions<'a> {
             target_rustdoc_args: None,
             target_rustc_args: None,
             local_rustdoc_args: None,
+            rustdoc_document_private_items: false,
             export_dir: None,
         })
     }
@@ -102,7 +111,7 @@ impl Packages {
         Ok(match (all, exclude.len(), package.len()) {
             (false, 0, 0) => Packages::Default,
             (false, 0, _) => Packages::Packages(package),
-            (false, _, _) => failure::bail!("--exclude can only be used together with --all"),
+            (false, _, _) => anyhow::bail!("--exclude can only be used together with --workspace"),
             (true, 0, _) => Packages::All,
             (true, _, _) => Packages::OptOut(exclude),
         })
@@ -126,12 +135,16 @@ impl Packages {
                 if !opt_out.is_empty() {
                     ws.config().shell().warn(format!(
                         "excluded package(s) {} not found in workspace `{}`",
-                        opt_out.iter().map(|x| x.as_ref()).collect::<Vec<_>>().join(", "),
+                        opt_out
+                            .iter()
+                            .map(|x| x.as_ref())
+                            .collect::<Vec<_>>()
+                            .join(", "),
                         ws.root().display(),
                     ))?;
                 }
                 packages
-            },
+            }
             Packages::Packages(packages) if packages.is_empty() => {
                 vec![PackageIdSpec::from_package_id(ws.current()?.package_id())]
             }
@@ -147,13 +160,13 @@ impl Packages {
         };
         if specs.is_empty() {
             if ws.is_virtual() {
-                failure::bail!(
+                anyhow::bail!(
                     "manifest path `{}` contains no package: The manifest is virtual, \
                      and the workspace has no members.",
                     ws.root().display()
                 )
             }
-            failure::bail!("no packages to compile")
+            anyhow::bail!("no packages to compile")
         }
         Ok(specs)
     }
@@ -172,7 +185,7 @@ impl Packages {
                     ws.members()
                         .find(|pkg| pkg.name().as_str() == name)
                         .ok_or_else(|| {
-                            failure::format_err!(
+                            anyhow::format_err!(
                                 "package `{}` is not a member of the workspace",
                                 name
                             )
@@ -182,6 +195,27 @@ impl Packages {
         };
         Ok(packages)
     }
+
+    /// Returns whether or not the user needs to pass a `-p` flag to target a
+    /// specific package in the workspace.
+    pub fn needs_spec_flag(&self, ws: &Workspace<'_>) -> bool {
+        match self {
+            Packages::Default => ws.default_members().count() > 1,
+            Packages::All => ws.members().count() > 1,
+            Packages::Packages(_) => true,
+            Packages::OptOut(_) => true,
+        }
+    }
+}
+
+#[derive(Debug, PartialEq, Eq)]
+pub enum LibRule {
+    /// Include the library, fail if not present
+    True,
+    /// Include the library if present
+    Default,
+    /// Exclude the library
+    False,
 }
 
 #[derive(Debug)]
@@ -198,7 +232,7 @@ pub enum CompileFilter {
     },
     Only {
         all_targets: bool,
-        lib: bool,
+        lib: LibRule,
         bins: FilterRule,
         examples: FilterRule,
         tests: FilterRule,
@@ -222,12 +256,11 @@ pub fn compile_with_exec<'a>(
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
     ws.emit_warnings()?;
-    compile_ws(ws, None, options, exec)
+    compile_ws(ws, options, exec)
 }
 
 pub fn compile_ws<'a>(
     ws: &Workspace<'a>,
-    source: Option<Box<dyn Source + 'a>>,
     options: &CompileOptions<'a>,
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
@@ -242,31 +275,79 @@ pub fn compile_ws<'a>(
         ref target_rustdoc_args,
         ref target_rustc_args,
         ref local_rustdoc_args,
+        rustdoc_document_private_items,
         ref export_dir,
     } = *options;
 
-    let default_arch_kind = if build_config.requested_target.is_some() {
-        Kind::Target
-    } else {
-        Kind::Host
-    };
+    match build_config.mode {
+        CompileMode::Test
+        | CompileMode::Build
+        | CompileMode::Check { .. }
+        | CompileMode::Bench
+        | CompileMode::RunCustomBuild => {
+            if std::env::var("RUST_FLAGS").is_ok() {
+                config.shell().warn(
+                    "Cargo does not read `RUST_FLAGS` environment variable. Did you mean `RUSTFLAGS`?",
+                )?;
+            }
+        }
+        CompileMode::Doc { .. } | CompileMode::Doctest => {
+            if std::env::var("RUSTDOC_FLAGS").is_ok() {
+                config.shell().warn(
+                    "Cargo does not read `RUSTDOC_FLAGS` environment variable. Did you mean `RUSTDOCFLAGS`?"
+                )?;
+            }
+        }
+    }
+
+    let profiles = Profiles::new(
+        ws.profiles(),
+        config,
+        build_config.requested_profile,
+        ws.features(),
+    )?;
 
     let specs = spec.to_package_id_specs(ws)?;
-    let features = Method::split_features(features);
-    let method = Method::Required {
-        dev_deps: ws.require_optional_deps() || filter.need_dev_deps(build_config.mode),
-        features: &features,
-        all_features,
-        uses_default_features: !no_default_features,
-    };
-    let resolve = ops::resolve_ws_with_method(ws, source, method, &specs)?;
-    let (packages, resolve_with_overrides) = resolve;
+    let dev_deps = ws.require_optional_deps() || filter.need_dev_deps(build_config.mode);
+    let opts = ResolveOpts::new(dev_deps, features, all_features, !no_default_features);
+    let resolve = ops::resolve_ws_with_opts(ws, opts, &specs)?;
+    let WorkspaceResolve {
+        mut pkg_set,
+        workspace_resolve,
+        targeted_resolve: resolve,
+    } = resolve;
 
+    let std_resolve = if let Some(crates) = &config.cli_unstable().build_std {
+        if build_config.build_plan {
+            config
+                .shell()
+                .warn("-Zbuild-std does not currently fully support --build-plan")?;
+        }
+        if build_config.requested_kind.is_host() {
+            // TODO: This should eventually be fixed. Unfortunately it is not
+            // easy to get the host triple in BuildConfig. Consider changing
+            // requested_target to an enum, or some other approach.
+            anyhow::bail!("-Zbuild-std requires --target");
+        }
+        let (mut std_package_set, std_resolve) = standard_lib::resolve_std(ws, crates)?;
+        remove_dylib_crate_type(&mut std_package_set)?;
+        pkg_set.add_set(std_package_set);
+        Some(std_resolve)
+    } else {
+        None
+    };
+
+    // Find the packages in the resolver that the user wants to build (those
+    // passed in with `-p` or the defaults from the workspace), and convert
+    // Vec<PackageIdSpec> to a Vec<&PackageId>.
     let to_build_ids = specs
         .iter()
-        .map(|s| s.query(resolve_with_overrides.iter()))
+        .map(|s| s.query(resolve.iter()))
         .collect::<CargoResult<Vec<_>>>()?;
-    let mut to_builds = packages.get_many(to_build_ids)?;
+    // Now get the `Package` for each `PackageId`. This may trigger a download
+    // if the user specified `-p` for a dependency that is not downloaded.
+    // Dependencies will be downloaded during build_unit_dependencies.
+    let mut to_builds = pkg_set.get_many(to_build_ids)?;
 
     // The ordering here affects some error messages coming out of cargo, so
     // let's be test and CLI friendly by always printing in the same order if
@@ -274,13 +355,13 @@ pub fn compile_ws<'a>(
     to_builds.sort_by_key(|p| p.package_id());
 
     for pkg in to_builds.iter() {
-        pkg.manifest().print_teapot(ws.config());
+        pkg.manifest().print_teapot(config);
 
         if build_config.mode.is_any_test()
             && !ws.is_member(pkg)
             && pkg.dependencies().iter().any(|dep| !dep.is_transitive())
         {
-            failure::bail!(
+            anyhow::bail!(
                 "package `{}` cannot be tested because it requires dev-dependencies \
                  and is not a member of the workspace",
                 pkg.name()
@@ -301,52 +382,91 @@ pub fn compile_ws<'a>(
         );
     }
 
-    let profiles = ws.profiles();
-    profiles.validate_packages(&mut config.shell(), &packages)?;
-
-    let units = generate_targets(
-        ws,
-        profiles,
-        &to_builds,
-        filter,
-        default_arch_kind,
-        &resolve_with_overrides,
-        build_config,
+    profiles.validate_packages(
+        ws.profiles(),
+        &mut config.shell(),
+        workspace_resolve.as_ref().unwrap_or(&resolve),
     )?;
 
-    let mut extra_compiler_args = HashMap::new();
+    let interner = UnitInterner::new();
+    let mut bcx = BuildContext::new(
+        ws,
+        &pkg_set,
+        config,
+        build_config,
+        profiles,
+        &interner,
+        HashMap::new(),
+    )?;
+    let units = generate_targets(
+        ws,
+        &to_builds,
+        filter,
+        build_config.requested_kind,
+        &resolve,
+        &bcx,
+    )?;
+
+    let std_roots = if let Some(crates) = &config.cli_unstable().build_std {
+        // Only build libtest if it looks like it is needed.
+        let mut crates = crates.clone();
+        if !crates.iter().any(|c| c == "test")
+            && units
+                .iter()
+                .any(|unit| unit.mode.is_rustc_test() && unit.target.harness())
+        {
+            // Only build libtest when libstd is built (libtest depends on libstd)
+            if crates.iter().any(|c| c == "std") {
+                crates.push("test".to_string());
+            }
+        }
+        standard_lib::generate_std_roots(
+            &bcx,
+            &crates,
+            std_resolve.as_ref().unwrap(),
+            build_config.requested_kind,
+        )?
+    } else {
+        Vec::new()
+    };
+
     if let Some(args) = extra_args {
         if units.len() != 1 {
-            failure::bail!(
+            anyhow::bail!(
                 "extra arguments to `{}` can only be passed to one \
                  target, consider filtering\nthe package by passing, \
                  e.g., `--lib` or `--bin NAME` to specify a single target",
                 extra_args_name
             );
         }
-        extra_compiler_args.insert(units[0], args);
+        bcx.extra_compiler_args.insert(units[0], args);
     }
-    if let Some(args) = local_rustdoc_args {
-        for unit in &units {
-            if unit.mode.is_doc() {
-                extra_compiler_args.insert(*unit, args.clone());
+    for unit in &units {
+        if unit.mode.is_doc() || unit.mode.is_doc_test() {
+            let mut extra_args = local_rustdoc_args.clone();
+
+            // Add `--document-private-items` rustdoc flag if requested or if
+            // the target is a binary. Binary crates get their private items
+            // documented by default.
+            if rustdoc_document_private_items || unit.target.is_bin() {
+                let mut args = extra_args.take().unwrap_or_else(|| vec![]);
+                args.push("--document-private-items".into());
+                extra_args = Some(args);
+            }
+
+            if let Some(args) = extra_args {
+                bcx.extra_compiler_args.insert(*unit, args.clone());
             }
         }
     }
 
+    let unit_dependencies =
+        build_unit_dependencies(&bcx, &resolve, std_resolve.as_ref(), &units, &std_roots)?;
+
     let ret = {
         let _p = profile::start("compiling");
-        let bcx = BuildContext::new(
-            ws,
-            &resolve_with_overrides,
-            &packages,
-            config,
-            &build_config,
-            profiles,
-            extra_compiler_args,
-        )?;
-        let cx = Context::new(config, &bcx)?;
-        cx.compile(&units, export_dir.clone(), &exec)?
+        let cx = Context::new(config, &bcx, unit_dependencies, build_config.requested_kind)?;
+        cx.compile(&units, export_dir.clone(), exec)?
     };
 
     Ok(ret)
@@ -361,6 +481,10 @@ impl FilterRule {
         }
     }
 
+    pub fn none() -> FilterRule {
+        FilterRule::Just(Vec::new())
+    }
+
     fn matches(&self, target: &Target) -> bool {
         match *self {
             FilterRule::All => true,
@@ -384,7 +508,8 @@ impl FilterRule {
 }
 
 impl CompileFilter {
-    pub fn new(
+    /// Construct a CompileFilter from raw command line arguments.
+    pub fn from_raw_arguments(
         lib_only: bool,
         bins: Vec<String>,
         all_bins: bool,
@@ -396,21 +521,31 @@ impl CompileFilter {
         all_bens: bool,
         all_targets: bool,
     ) -> CompileFilter {
+        if all_targets {
+            return CompileFilter::new_all_targets();
+        }
+        let rule_lib = if lib_only {
+            LibRule::True
+        } else {
+            LibRule::False
+        };
         let rule_bins = FilterRule::new(bins, all_bins);
         let rule_tsts = FilterRule::new(tsts, all_tsts);
         let rule_exms = FilterRule::new(exms, all_exms);
         let rule_bens = FilterRule::new(bens, all_bens);
 
-        if all_targets {
-            CompileFilter::Only {
-                all_targets: true,
-                lib: true,
-                bins: FilterRule::All,
-                examples: FilterRule::All,
-                benches: FilterRule::All,
-                tests: FilterRule::All,
-            }
-        } else if lib_only
+        CompileFilter::new(rule_lib, rule_bins, rule_tsts, rule_exms, rule_bens)
+    }
+
+    /// Construct a CompileFilter from underlying primitives.
+    pub fn new(
+        rule_lib: LibRule,
+        rule_bins: FilterRule,
+        rule_tsts: FilterRule,
+        rule_exms: FilterRule,
+        rule_bens: FilterRule,
+    ) -> CompileFilter {
+        if rule_lib == LibRule::True
             || rule_bins.is_specific()
             || rule_tsts.is_specific()
             || rule_exms.is_specific()
@@ -418,7 +553,7 @@ impl CompileFilter {
         {
             CompileFilter::Only {
                 all_targets: false,
-                lib: lib_only,
+                lib: rule_lib,
                 bins: rule_bins,
                 examples: rule_exms,
                 benches: rule_bens,
@@ -431,6 +566,17 @@ impl CompileFilter {
         }
     }
 
+    pub fn new_all_targets() -> CompileFilter {
+        CompileFilter::Only {
+            all_targets: true,
+            lib: LibRule::Default,
+            bins: FilterRule::All,
+            examples: FilterRule::All,
+            benches: FilterRule::All,
+            tests: FilterRule::All,
+        }
+    }
+
     pub fn need_dev_deps(&self, mode: CompileMode) -> bool {
         match mode {
             CompileMode::Test | CompileMode::Doctest | CompileMode::Bench => true,
@@ -454,7 +600,7 @@ impl CompileFilter {
         match *self {
             CompileFilter::Default { .. } => true,
             CompileFilter::Only {
-                lib,
+                ref lib,
                 ref bins,
                 ref examples,
                 ref tests,
@@ -466,7 +612,13 @@ impl CompileFilter {
                     TargetKind::Test => tests,
                     TargetKind::Bench => benches,
                     TargetKind::ExampleBin | TargetKind::ExampleLib(..) => examples,
-                    TargetKind::Lib(..) => return lib,
+                    TargetKind::Lib(..) => {
+                        return match *lib {
+                            LibRule::True => true,
+                            LibRule::Default => true,
+                            LibRule::False => false,
+                        };
+                    }
                     TargetKind::CustomBuild => return false,
                 };
                 rule.matches(target)
@@ -502,16 +654,15 @@ struct Proposal<'a> {
 /// compile. Dependencies for these targets are computed later in `unit_dependencies`.
 fn generate_targets<'a>(
     ws: &Workspace<'_>,
-    profiles: &Profiles,
     packages: &[&'a Package],
     filter: &CompileFilter,
-    default_arch_kind: Kind,
-    resolve: &Resolve,
-    build_config: &BuildConfig,
+    default_arch_kind: CompileKind,
+    resolve: &'a Resolve,
+    bcx: &BuildContext<'a, '_>,
 ) -> CargoResult<Vec<Unit<'a>>> {
     // Helper for creating a `Unit` struct.
     let new_unit = |pkg: &'a Package, target: &'a Target, target_mode: CompileMode| {
-        let unit_for = if build_config.mode.is_any_test() {
+        let unit_for = if target_mode.is_any_test() {
             // NOTE: the `UnitFor` here is subtle. If you have a profile
             // with `panic` set, the `panic` flag is cleared for
             // tests/benchmarks and their dependencies. If this
@@ -530,7 +681,7 @@ fn generate_targets<'a>(
             //
             // Forcing the lib to be compiled three times during `cargo
             // test` is probably also not desirable.
-            UnitFor::new_test()
+            UnitFor::new_test(bcx.config)
         } else if target.for_host() {
             // Proc macro / plugin should not have `panic` set.
             UnitFor::new_compiler()
@@ -565,26 +716,20 @@ fn generate_targets<'a>(
             CompileMode::Bench => CompileMode::Test,
             _ => target_mode,
         };
-        // Plugins or proc macros should be built for the host.
-        let kind = if target.for_host() {
-            Kind::Host
-        } else {
-            default_arch_kind
-        };
-        let profile = profiles.get_profile(
-            pkg.package_id(),
-            ws.is_member(pkg),
-            unit_for,
-            target_mode,
-            build_config.release,
-        );
-        Unit {
+        let kind = default_arch_kind.for_target(target);
+        let profile =
+            bcx.profiles
+                .get_profile(pkg.package_id(), ws.is_member(pkg), unit_for, target_mode);
+        let features = resolve.features_sorted(pkg.package_id());
+        bcx.units.intern(
             pkg,
             target,
             profile,
             kind,
-            mode: target_mode,
-        }
+            target_mode,
+            features,
+            /*is_std*/ false,
+        )
     };
 
     // Create a list of proposed targets.
@@ -595,14 +740,14 @@ fn generate_targets<'a>(
             required_features_filterable,
         } => {
             for pkg in packages {
-                let default = filter_default_targets(pkg.targets(), build_config.mode);
+                let default = filter_default_targets(pkg.targets(), bcx.build_config.mode);
                 proposals.extend(default.into_iter().map(|target| Proposal {
                     pkg,
                     target,
                     requires_features: !required_features_filterable,
-                    mode: build_config.mode,
+                    mode: bcx.build_config.mode,
                 }));
-                if build_config.mode == CompileMode::Test {
+                if bcx.build_config.mode == CompileMode::Test {
                     if let Some(t) = pkg
                         .targets()
                         .iter()
@@ -620,17 +765,19 @@ fn generate_targets<'a>(
         }
         CompileFilter::Only {
             all_targets,
-            lib,
+            ref lib,
             ref bins,
             ref examples,
             ref tests,
             ref benches,
         } => {
-            if lib {
+            if *lib != LibRule::False {
                 let mut libs = Vec::new();
-                for proposal in filter_targets(packages, Target::is_lib, false, build_config.mode) {
+                for proposal in
+                    filter_targets(packages, Target::is_lib, false, bcx.build_config.mode)
+                {
                     let Proposal { target, pkg, .. } = proposal;
-                    if build_config.mode == CompileMode::Doctest && !target.doctestable() {
+                    if bcx.build_config.mode.is_doc_test() && !target.doctestable() {
                         ws.config().shell().warn(format!(
                             "doc tests are not supported for crate type(s) `{}` in package `{}`",
                             target.rustc_crate_types().join(", "),
@@ -640,15 +787,12 @@ fn generate_targets<'a>(
                         libs.push(proposal)
                     }
                 }
-                if !all_targets && libs.is_empty() {
+                if !all_targets && libs.is_empty() && *lib == LibRule::True {
                     let names = packages.iter().map(|pkg| pkg.name()).collect::<Vec<_>>();
                     if names.len() == 1 {
-                        failure::bail!("no library targets found in package `{}`", names[0]);
+                        anyhow::bail!("no library targets found in package `{}`", names[0]);
                     } else {
-                        failure::bail!(
-                            "no library targets found in packages: {}",
-                            names.join(", ")
-                        );
+                        anyhow::bail!("no library targets found in packages: {}", names.join(", "));
                     }
                 }
                 proposals.extend(libs);
@@ -660,10 +804,10 @@ fn generate_targets<'a>(
                 FilterRule::All => Target::tested,
                 FilterRule::Just(_) => Target::is_test,
             };
-            let test_mode = match build_config.mode {
+            let test_mode = match bcx.build_config.mode {
                 CompileMode::Build => CompileMode::Test,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => build_config.mode,
+                _ => bcx.build_config.mode,
             };
             // If `--benches` was specified, add all targets that would be
             // generated by `cargo bench`.
@@ -671,10 +815,10 @@ fn generate_targets<'a>(
                 FilterRule::All => Target::benched,
                 FilterRule::Just(_) => Target::is_bench,
             };
-            let bench_mode = match build_config.mode {
+            let bench_mode = match bcx.build_config.mode {
                 CompileMode::Build => CompileMode::Bench,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => build_config.mode,
+                _ => bcx.build_config.mode,
             };
 
             proposals.extend(list_rule_targets(
@@ -682,14 +826,14 @@ fn generate_targets<'a>(
                 bins,
                 "bin",
                 Target::is_bin,
-                build_config.mode,
+                bcx.build_config.mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
                 examples,
                 "example",
                 Target::is_example,
-                build_config.mode,
+                bcx.build_config.mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
@@ -737,7 +881,7 @@ fn generate_targets<'a>(
                 .iter()
                 .map(|s| format!("`{}`", s))
                 .collect();
-            failure::bail!(
+            anyhow::bail!(
                 "target `{}` in package `{}` requires the features: {}\n\
                  Consider enabling them by passing, e.g., `--features=\"{}\"`",
                 target.name(),
@@ -835,26 +979,18 @@ fn find_named_targets<'a>(
     let filter = |t: &Target| t.name() == target_name && is_expected_kind(t);
     let proposals = filter_targets(packages, filter, true, mode);
     if proposals.is_empty() {
-        let suggestion = packages
-            .iter()
-            .flat_map(|pkg| {
-                pkg.targets()
-                    .iter()
-                    .filter(|target| is_expected_kind(target))
-            })
-            .map(|target| (lev_distance(target_name, target.name()), target))
-            .filter(|&(d, _)| d < 4)
-            .min_by_key(|t| t.0)
-            .map(|t| t.1);
-        match suggestion {
-            Some(s) => failure::bail!(
-                "no {} target named `{}`\n\nDid you mean `{}`?",
-                target_desc,
-                target_name,
-                s.name()
-            ),
-            None => failure::bail!("no {} target named `{}`", target_desc, target_name),
-        }
+        let targets = packages.iter().flat_map(|pkg| {
+            pkg.targets()
+                .iter()
+                .filter(|target| is_expected_kind(target))
+        });
+        let suggestion = closest_msg(target_name, targets, |t| t.name());
+        anyhow::bail!(
+            "no {} target named `{}`{}",
+            target_desc,
+            target_name,
+            suggestion
+        );
     }
     Ok(proposals)
 }
@@ -878,3 +1014,35 @@ fn filter_targets<'a>(
     }
     proposals
 }
+
+/// When using `-Zbuild-std` we're building the standard library, but a
+/// technical detail of the standard library right now is that it builds itself
+/// as both an `rlib` and a `dylib`. We don't actually want to really publicize
+/// the `dylib` and in general it's a pain to work with, so when building libstd
+/// we want to remove the `dylib` crate type.
+///
+/// Cargo doesn't have a fantastic way of doing that right now, so let's hack
+/// around it a bit and (ab)use the fact that we have mutable access to
+/// `PackageSet` here to rewrite downloaded packages. We iterate over all `path`
+/// packages (which should download immediately and not actually cause blocking
+/// here) and edit their manifests to only list one `LibKind` for an `Rlib`.
+fn remove_dylib_crate_type(set: &mut PackageSet<'_>) -> CargoResult<()> {
+    let ids = set
+        .package_ids()
+        .filter(|p| p.source_id().is_path())
+        .collect::<Vec<_>>();
+    set.get_many(ids.iter().cloned())?;
+
+    for id in ids {
+        let pkg = set.lookup_mut(id).expect("should be downloaded now");
+
+        for target in pkg.manifest_mut().targets_mut() {
+            if let TargetKind::Lib(crate_types) = target.kind_mut() {
+                crate_types.truncate(0);
+                crate_types.push(LibKind::Rlib);
+            }
+        }
+    }
+
+    Ok(())
+}
