@@ -1,21 +1,21 @@
+use crate::core::GitReference;
+use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::paths;
+use crate::util::process_builder::process;
+use crate::util::{internal, network, Config, IntoUrl, Progress};
+use curl::easy::{Easy, List};
+use git2::{self, ObjectType};
+use log::{debug, info};
+use serde::ser;
+use serde::Serialize;
 use std::env;
 use std::fmt;
-use std::fs::{self, File};
+use std::fs::File;
 use std::mem;
 use std::path::{Path, PathBuf};
 use std::process::Command;
-
-use curl::easy::{Easy, List};
-use git2::{self, ObjectType};
-use serde::ser;
 use url::Url;
 
-use core::GitReference;
-use util::errors::{CargoError, CargoResult, CargoResultExt};
-use util::paths;
-use util::process_builder::process;
-use util::{internal, network, Config, Progress, ToUrl};
-
 #[derive(PartialEq, Clone, Debug)]
 pub struct GitRevision(git2::Oid);
 
@@ -34,7 +34,7 @@ where
 }
 
 impl fmt::Display for GitRevision {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::Display::fmt(&self.0, f)
     }
 }
@@ -67,7 +67,7 @@ pub struct GitDatabase {
 
 /// `GitCheckout` is a local checkout of a particular revision. Calling
 /// `clone_into` with a reference will resolve the reference into a revision,
-/// and return a `CargoError` if no revision for that reference was found.
+/// and return a `anyhow::Error` if no revision for that reference was found.
 #[derive(Serialize)]
 pub struct GitCheckout<'a> {
     database: &'a GitDatabase,
@@ -109,7 +109,8 @@ impl GitRemote {
         let (repo, rev) = match repo_and_rev {
             Some(pair) => pair,
             None => {
-                let repo = self.clone_into(into, cargo_config)
+                let repo = self
+                    .clone_into(into, cargo_config)
                     .chain_err(|| format!("failed to clone into: {}", into.display()))?;
                 let rev = reference.resolve(&repo)?;
                 (repo, rev)
@@ -138,18 +139,18 @@ impl GitRemote {
     fn fetch_into(&self, dst: &mut git2::Repository, cargo_config: &Config) -> CargoResult<()> {
         // Create a local anonymous remote in the repository to fetch the url
         let refspec = "refs/heads/*:refs/heads/*";
-        fetch(dst, &self.url, refspec, cargo_config)
+        fetch(dst, self.url.as_str(), refspec, cargo_config)
     }
 
     fn clone_into(&self, dst: &Path, cargo_config: &Config) -> CargoResult<git2::Repository> {
-        if fs::metadata(&dst).is_ok() {
+        if dst.exists() {
             paths::remove_dir_all(dst)?;
         }
-        fs::create_dir_all(dst)?;
-        let mut repo = git2::Repository::init_bare(dst)?;
+        paths::create_dir_all(dst)?;
+        let mut repo = init(dst, true)?;
         fetch(
             &mut repo,
-            &self.url,
+            self.url.as_str(),
             "refs/heads/*:refs/heads/*",
             cargo_config,
         )?;
@@ -163,7 +164,7 @@ impl GitDatabase {
         rev: GitRevision,
         dest: &Path,
         cargo_config: &Config,
-    ) -> CargoResult<GitCheckout> {
+    ) -> CargoResult<GitCheckout<'_>> {
         let mut checkout = None;
         if let Ok(repo) = git2::Repository::open(dest) {
             let mut co = GitCheckout::new(dest, self, rev.clone(), repo);
@@ -211,13 +212,14 @@ impl GitReference {
                 let obj = obj.peel(ObjectType::Commit)?;
                 Ok(obj.id())
             })()
-                .chain_err(|| format!("failed to find tag `{}`", s))?,
+            .chain_err(|| format!("failed to find tag `{}`", s))?,
             GitReference::Branch(ref s) => {
-                let b = repo.find_branch(s, git2::BranchType::Local)
+                let b = repo
+                    .find_branch(s, git2::BranchType::Local)
                     .chain_err(|| format!("failed to find branch `{}`", s))?;
                 b.get()
                     .target()
-                    .ok_or_else(|| format_err!("branch `{}` did not have a target", s))?
+                    .ok_or_else(|| anyhow::format_err!("branch `{}` did not have a target", s))?
             }
             GitReference::Rev(ref s) => {
                 let obj = repo.revparse_single(s)?;
@@ -253,7 +255,7 @@ impl<'a> GitCheckout<'a> {
         config: &Config,
     ) -> CargoResult<GitCheckout<'a>> {
         let dirname = into.parent().unwrap();
-        fs::create_dir_all(&dirname).chain_err(|| format!("Couldn't mkdir {}", dirname.display()))?;
+        paths::create_dir_all(&dirname)?;
         if into.exists() {
             paths::remove_dir_all(into)?;
         }
@@ -269,9 +271,9 @@ impl<'a> GitCheckout<'a> {
         //
         // Note that we still use the same fetch options because while we don't
         // need authentication information we may want progress bars and such.
-        let url = database.path.to_url()?;
+        let url = database.path.into_url()?;
         let mut repo = None;
-        with_fetch_options(&git_config, &url, config, &mut |fopts| {
+        with_fetch_options(&git_config, url.as_str(), config, &mut |fopts| {
             let mut checkout = git2::build::CheckoutBuilder::new();
             checkout.dry_run(); // we'll do this below during a `reset`
 
@@ -305,9 +307,9 @@ impl<'a> GitCheckout<'a> {
 
     fn fetch(&mut self, cargo_config: &Config) -> CargoResult<()> {
         info!("fetch {}", self.repo.path().display());
-        let url = self.database.path.to_url()?;
+        let url = self.database.path.into_url()?;
         let refspec = "refs/heads/*:refs/heads/*";
-        fetch(&mut self.repo, &url, refspec, cargo_config)?;
+        fetch(&mut self.repo, url.as_str(), refspec, cargo_config)?;
         Ok(())
     }
 
@@ -316,7 +318,7 @@ impl<'a> GitCheckout<'a> {
     }
 
     fn reset(&self, config: &Config) -> CargoResult<()> {
-        // If we're interrupted while performing this reset (e.g. we die because
+        // If we're interrupted while performing this reset (e.g., we die because
         // of a signal) Cargo needs to be sure to try to check out this repo
         // again on the next go-round.
         //
@@ -352,7 +354,7 @@ impl<'a> GitCheckout<'a> {
 
         fn update_submodule(
             parent: &git2::Repository,
-            child: &mut git2::Submodule,
+            child: &mut git2::Submodule<'_>,
             cargo_config: &Config,
         ) -> CargoResult<()> {
             child.init(false)?;
@@ -385,14 +387,12 @@ impl<'a> GitCheckout<'a> {
                 Err(..) => {
                     let path = parent.workdir().unwrap().join(child.path());
                     let _ = paths::remove_dir_all(&path);
-                    git2::Repository::init(&path)?
+                    init(&path, false)?
                 }
             };
-
             // Fetch data from origin and reset to the head commit
             let refspec = "refs/heads/*:refs/heads/*";
-            let url = url.to_url()?;
-            fetch(&mut repo, &url, refspec, cargo_config).chain_err(|| {
+            fetch(&mut repo, url, refspec, cargo_config).chain_err(|| {
                 internal(format!(
                     "failed to fetch submodule `{}` from {}",
                     child.name().unwrap_or(""),
@@ -424,7 +424,7 @@ impl<'a> GitCheckout<'a> {
 ///
 /// * If a username/password is allowed, then we fallback to git2-rs's
 ///   implementation of the credential helper. This is what is configured
-///   with `credential.helper` in git, and is the interface for the OSX
+///   with `credential.helper` in git, and is the interface for the macOS
 ///   keychain, for example.
 ///
 /// * After the above two have failed, we just kinda grapple attempting to
@@ -436,7 +436,7 @@ impl<'a> GitCheckout<'a> {
 /// attempted and we don't try the same ones again.
 fn with_authentication<T, F>(url: &str, cfg: &git2::Config, mut f: F) -> CargoResult<T>
 where
-    F: FnMut(&mut git2::Credentials) -> CargoResult<T>,
+    F: FnMut(&mut git2::Credentials<'_>) -> CargoResult<T>,
 {
     let mut cred_helper = git2::CredentialHelper::new(url);
     cred_helper.config(cfg);
@@ -479,7 +479,7 @@ where
         // ssh-agent currently.
         //
         // If we get called with this then the only way that should be possible
-        // is if a username is specified in the URL itself (e.g. `username` is
+        // is if a username is specified in the URL itself (e.g., `username` is
         // Some), hence the unwrap() here. We try custom usernames down below.
         if allowed.contains(git2::CredentialType::SSH_KEY) && !tried_sshkey {
             // If ssh-agent authentication fails, libgit2 will keep
@@ -498,7 +498,13 @@ where
         // but we currently don't! Right now the only way we support fetching a
         // plaintext password is through the `credential.helper` support, so
         // fetch that here.
-        if allowed.contains(git2::CredentialType::USER_PASS_PLAINTEXT) {
+        //
+        // If ssh-agent authentication fails, libgit2 will keep calling this
+        // callback asking for other authentication methods to try. Check
+        // cred_helper_bad to make sure we only try the git credentail helper
+        // once, to avoid looping forever.
+        if allowed.contains(git2::CredentialType::USER_PASS_PLAINTEXT) && cred_helper_bad.is_none()
+        {
             let r = git2::Cred::credential_helper(cfg, url, username);
             cred_helper_bad = Some(r.is_err());
             return r;
@@ -566,7 +572,7 @@ where
             //    for another mode of authentication.
             //
             // Essentially, if `attempts == 2` then in theory the only error was
-            // that this username failed to authenticate (e.g. no other network
+            // that this username failed to authenticate (e.g., no other network
             // errors happened). Otherwise something else is funny so we bail
             // out.
             if attempts != 2 {
@@ -582,7 +588,7 @@ where
     // In the case of an authentication failure (where we tried something) then
     // we try to give a more helpful error message about precisely what we
     // tried.
-    let res = res.map_err(CargoError::from).chain_err(|| {
+    let res = res.map_err(anyhow::Error::from).chain_err(|| {
         let mut msg = "failed to authenticate when downloading \
                        repository"
             .to_string();
@@ -617,7 +623,7 @@ where
     Ok(res)
 }
 
-fn reset(repo: &git2::Repository, obj: &git2::Object, config: &Config) -> CargoResult<()> {
+fn reset(repo: &git2::Repository, obj: &git2::Object<'_>, config: &Config) -> CargoResult<()> {
     let mut pb = Progress::new("Checkout", config);
     let mut opts = git2::build::CheckoutBuilder::new();
     opts.progress(|_, cur, max| {
@@ -629,13 +635,13 @@ fn reset(repo: &git2::Repository, obj: &git2::Object, config: &Config) -> CargoR
 
 pub fn with_fetch_options(
     git_config: &git2::Config,
-    url: &Url,
+    url: &str,
     config: &Config,
-    cb: &mut FnMut(git2::FetchOptions) -> CargoResult<()>,
+    cb: &mut dyn FnMut(git2::FetchOptions<'_>) -> CargoResult<()>,
 ) -> CargoResult<()> {
     let mut progress = Progress::new("Fetch", config);
     network::with_retry(config, || {
-        with_authentication(url.as_str(), git_config, |f| {
+        with_authentication(url, git_config, |f| {
             let mut rcb = git2::RemoteCallbacks::new();
             rcb.credentials(f);
 
@@ -658,30 +664,33 @@ pub fn with_fetch_options(
 
 pub fn fetch(
     repo: &mut git2::Repository,
-    url: &Url,
+    url: &str,
     refspec: &str,
     config: &Config,
 ) -> CargoResult<()> {
     if config.frozen() {
-        bail!(
+        anyhow::bail!(
             "attempting to update a git repository, but --frozen \
              was specified"
         )
     }
     if !config.network_allowed() {
-        bail!("can't update a git repository in the offline mode")
+        anyhow::bail!("can't update a git repository in the offline mode")
     }
 
     // If we're fetching from GitHub, attempt GitHub's special fast path for
     // testing if we've already got an up-to-date copy of the repository
-    if url.host_str() == Some("github.com") {
-        if let Ok(oid) = repo.refname_to_id("refs/remotes/origin/master") {
-            let mut handle = config.http()?.borrow_mut();
-            debug!("attempting GitHub fast path for {}", url);
-            if github_up_to_date(&mut handle, url, &oid) {
-                return Ok(());
-            } else {
-                debug!("fast path failed, falling back to a git fetch");
+
+    if let Ok(url) = Url::parse(url) {
+        if url.host_str() == Some("github.com") {
+            if let Ok(oid) = repo.refname_to_id("refs/remotes/origin/master") {
+                let mut handle = config.http()?.borrow_mut();
+                debug!("attempting GitHub fast path for {}", url);
+                if github_up_to_date(&mut handle, &url, &oid) {
+                    return Ok(());
+                } else {
+                    debug!("fast path failed, falling back to a git fetch");
+                }
             }
         }
     }
@@ -692,16 +701,14 @@ pub fn fetch(
     // request we're about to issue.
     maybe_gc_repo(repo)?;
 
-    // Unfortuantely `libgit2` is notably lacking in the realm of authentication
+    // Unfortunately `libgit2` is notably lacking in the realm of authentication
     // when compared to the `git` command line. As a result, allow an escape
     // hatch for users that would prefer to use `git`-the-CLI for fetching
     // repositories instead of `libgit2`-the-library. This should make more
     // flavors of authentication possible while also still giving us all the
     // speed and portability of using `libgit2`.
-    if let Some(val) = config.get_bool("net.git-fetch-with-cli")? {
-        if val.val {
-            return fetch_with_cli(repo, url, refspec, config);
-        }
+    if let Some(true) = config.net_config()?.git_fetch_with_cli {
+        return fetch_with_cli(repo, url, refspec, config);
     }
 
     debug!("doing a fetch for {}", url);
@@ -711,7 +718,7 @@ pub fn fetch(
         // repository. It could also fail, however, for a whole slew of other
         // reasons (aka network related reasons). We want Cargo to automatically
         // recover from corrupt repositories, but we don't want Cargo to stomp
-        // over other legitimate errors.o
+        // over other legitimate errors.
         //
         // Consequently we save off the error of the `fetch` operation and if it
         // looks like a "corrupt repo" error then we blow away the repo and try
@@ -720,7 +727,8 @@ pub fn fetch(
         let mut repo_reinitialized = false;
         loop {
             debug!("initiating fetch of {} from {}", refspec, url);
-            let res = repo.remote_anonymous(url.as_str())?
+            let res = repo
+                .remote_anonymous(url)?
                 .fetch(&[refspec], Some(&mut opts), None);
             let err = match res {
                 Ok(()) => break,
@@ -747,19 +755,33 @@ pub fn fetch(
 
 fn fetch_with_cli(
     repo: &mut git2::Repository,
-    url: &Url,
+    url: &str,
     refspec: &str,
     config: &Config,
 ) -> CargoResult<()> {
     let mut cmd = process("git");
     cmd.arg("fetch")
         .arg("--tags") // fetch all tags
-        .arg("--quiet")
-        .arg(url.to_string())
+        .arg("--force") // handle force pushes
+        .arg("--update-head-ok") // see discussion in #2078
+        .arg(url)
         .arg(refspec)
+        // If cargo is run by git (for example, the `exec` command in `git
+        // rebase`), the GIT_DIR is set by git and will point to the wrong
+        // location (this takes precedence over the cwd). Make sure this is
+        // unset so git will look at cwd for the repo.
+        .env_remove("GIT_DIR")
+        // The reset of these may not be necessary, but I'm including them
+        // just to be extra paranoid and avoid any issues.
+        .env_remove("GIT_WORK_TREE")
+        .env_remove("GIT_INDEX_FILE")
+        .env_remove("GIT_OBJECT_DIRECTORY")
+        .env_remove("GIT_ALTERNATE_OBJECT_DIRECTORIES")
         .cwd(repo.path());
-    config.shell().verbose(|s| s.status("Running", &cmd.to_string()))?;
-    cmd.exec()?;
+    config
+        .shell()
+        .verbose(|s| s.status("Running", &cmd.to_string()))?;
+    cmd.exec_with_output()?;
     Ok(())
 }
 
@@ -833,7 +855,7 @@ fn reinitialize(repo: &mut git2::Repository) -> CargoResult<()> {
     debug!("reinitializing git repo at {:?}", path);
     let tmp = path.join("tmp");
     let bare = !repo.path().ends_with(".git");
-    *repo = git2::Repository::init(&tmp)?;
+    *repo = init(&tmp, false)?;
     for entry in path.read_dir()? {
         let entry = entry?;
         if entry.file_name().to_str() == Some("tmp") {
@@ -842,15 +864,21 @@ fn reinitialize(repo: &mut git2::Repository) -> CargoResult<()> {
         let path = entry.path();
         drop(paths::remove_file(&path).or_else(|_| paths::remove_dir_all(&path)));
     }
-    if bare {
-        *repo = git2::Repository::init_bare(path)?;
-    } else {
-        *repo = git2::Repository::init(path)?;
-    }
+    *repo = init(&path, bare)?;
     paths::remove_dir_all(&tmp)?;
     Ok(())
 }
 
+fn init(path: &Path, bare: bool) -> CargoResult<git2::Repository> {
+    let mut opts = git2::RepositoryInitOptions::new();
+    // Skip anything related to templates, they just call all sorts of issues as
+    // we really don't want to use them yet they insist on being used. See #6240
+    // for an example issue that comes up.
+    opts.external_template(false);
+    opts.bare(bare);
+    Ok(git2::Repository::init_opts(&path, &opts)?)
+}
+
 /// Updating the index is done pretty regularly so we want it to be as fast as
 /// possible. For registries hosted on GitHub (like the crates.io index) there's
 /// a fast path available to use [1] to tell us that there's no updates to be
@@ -868,18 +896,20 @@ fn reinitialize(repo: &mut git2::Repository) -> CargoResult<()> {
 /// just return a `bool`. Any real errors will be reported through the normal
 /// update path above.
 fn github_up_to_date(handle: &mut Easy, url: &Url, oid: &git2::Oid) -> bool {
-    macro_rules! try {
-        ($e:expr) => (match $e {
-            Some(e) => e,
-            None => return false,
-        })
+    macro_rules! r#try {
+        ($e:expr) => {
+            match $e {
+                Some(e) => e,
+                None => return false,
+            }
+        };
     }
 
     // This expects GitHub urls in the form `github.com/user/repo` and nothing
     // else
-    let mut pieces = try!(url.path_segments());
-    let username = try!(pieces.next());
-    let repo = try!(pieces.next());
+    let mut pieces = r#try!(url.path_segments());
+    let username = r#try!(pieces.next());
+    let repo = r#try!(pieces.next());
     if pieces.next().is_some() {
         return false;
     }
@@ -888,14 +918,14 @@ fn github_up_to_date(handle: &mut Easy, url: &Url, oid: &git2::Oid) -> bool {
         "https://api.github.com/repos/{}/{}/commits/master",
         username, repo
     );
-    try!(handle.get(true).ok());
-    try!(handle.url(&url).ok());
-    try!(handle.useragent("cargo").ok());
+    r#try!(handle.get(true).ok());
+    r#try!(handle.url(&url).ok());
+    r#try!(handle.useragent("cargo").ok());
     let mut headers = List::new();
-    try!(headers.append("Accept: application/vnd.github.3.sha").ok());
-    try!(headers.append(&format!("If-None-Match: \"{}\"", oid)).ok());
-    try!(handle.http_headers(headers).ok());
-    try!(handle.perform().ok());
+    r#try!(headers.append("Accept: application/vnd.github.3.sha").ok());
+    r#try!(headers.append(&format!("If-None-Match: \"{}\"", oid)).ok());
+    r#try!(handle.http_headers(headers).ok());
+    r#try!(handle.perform().ok());
 
-    try!(handle.response_code().ok()) == 304
+    r#try!(handle.response_code().ok()) == 304
 }
