@@ -1,15 +1,15 @@
+//! Tests for the `cargo doc` command.
+
 use std::fs::{self, File};
 use std::io::Read;
 use std::str;
-use support;
 
-use glob::glob;
-use support::paths::CargoPathExt;
-use support::registry::Package;
-use support::{basic_lib_manifest, basic_manifest, git, project};
-use support::{is_nightly, rustc_host};
+use cargo_test_support::paths::CargoPathExt;
+use cargo_test_support::registry::Package;
+use cargo_test_support::{basic_lib_manifest, basic_manifest, git, project};
+use cargo_test_support::{is_nightly, rustc_host};
 
-#[test]
+#[cargo_test]
 fn simple() {
     let p = project()
         .file(
@@ -21,7 +21,8 @@ fn simple() {
             authors = []
             build = "build.rs"
         "#,
-        ).file("build.rs", "fn main() {}")
+        )
+        .file("build.rs", "fn main() {}")
         .file("src/lib.rs", "pub fn foo() {}")
         .build();
 
@@ -32,12 +33,13 @@ fn simple() {
 [..] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
     assert!(p.root().join("target/doc").is_dir());
     assert!(p.root().join("target/doc/foo/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_no_libs() {
     let p = project()
         .file(
@@ -52,13 +54,14 @@ fn doc_no_libs() {
             name = "foo"
             doc = false
         "#,
-        ).file("src/main.rs", "bad code")
+        )
+        .file("src/main.rs", "bad code")
         .build();
 
     p.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_twice() {
     let p = project().file("src/lib.rs", "pub fn foo() {}").build();
 
@@ -68,12 +71,13 @@ fn doc_twice() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 
     p.cargo("doc").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_deps() {
     let p = project()
         .file(
@@ -87,7 +91,8 @@ fn doc_deps() {
             [dependencies.bar]
             path = "bar"
         "#,
-        ).file("src/lib.rs", "extern crate bar; pub fn foo() {}")
+        )
+        .file("src/lib.rs", "extern crate bar; pub fn foo() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
@@ -100,32 +105,19 @@ fn doc_deps() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 
     assert!(p.root().join("target/doc").is_dir());
     assert!(p.root().join("target/doc/foo/index.html").is_file());
     assert!(p.root().join("target/doc/bar/index.html").is_file());
 
     // Verify that it only emits rmeta for the dependency.
-    assert_eq!(
-        glob(&p.root().join("target/debug/**/*.rlib").to_str().unwrap())
-            .unwrap()
-            .count(),
-        0
-    );
-    assert_eq!(
-        glob(
-            &p.root()
-                .join("target/debug/deps/libbar-*.rmeta")
-                .to_str()
-                .unwrap()
-        ).unwrap()
-        .count(),
-        1
-    );
+    assert_eq!(p.glob("target/debug/**/*.rlib").count(), 0);
+    assert_eq!(p.glob("target/debug/deps/libbar-*.rmeta").count(), 1);
 
     p.cargo("doc")
-        .env("RUST_LOG", "cargo::ops::cargo_rustc::fingerprint")
+        .env("CARGO_LOG", "cargo::ops::cargo_rustc::fingerprint")
         .with_stdout("")
         .run();
 
@@ -134,7 +126,7 @@ fn doc_deps() {
     assert!(p.root().join("target/doc/bar/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_no_deps() {
     let p = project()
         .file(
@@ -148,7 +140,8 @@ fn doc_no_deps() {
             [dependencies.bar]
             path = "bar"
         "#,
-        ).file("src/lib.rs", "extern crate bar; pub fn foo() {}")
+        )
+        .file("src/lib.rs", "extern crate bar; pub fn foo() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
@@ -160,14 +153,15 @@ fn doc_no_deps() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 
     assert!(p.root().join("target/doc").is_dir());
     assert!(p.root().join("target/doc/foo/index.html").is_file());
     assert!(!p.root().join("target/doc/bar/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_only_bin() {
     let p = project()
         .file(
@@ -181,7 +175,8 @@ fn doc_only_bin() {
             [dependencies.bar]
             path = "bar"
         "#,
-        ).file("src/main.rs", "extern crate bar; pub fn foo() {}")
+        )
+        .file("src/main.rs", "extern crate bar; pub fn foo() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
@@ -193,7 +188,7 @@ fn doc_only_bin() {
     assert!(p.root().join("target/doc/foo/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_multiple_targets_same_name_lib() {
     let p = project()
         .file(
@@ -202,7 +197,8 @@ fn doc_multiple_targets_same_name_lib() {
             [workspace]
             members = ["foo", "bar"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -211,7 +207,8 @@ fn doc_multiple_targets_same_name_lib() {
             [lib]
             name = "foo_lib"
         "#,
-        ).file("foo/src/lib.rs", "")
+        )
+        .file("foo/src/lib.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -221,10 +218,11 @@ fn doc_multiple_targets_same_name_lib() {
             [lib]
             name = "foo_lib"
         "#,
-        ).file("bar/src/lib.rs", "")
+        )
+        .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_status(101)
         .with_stderr_contains("[..] library `foo_lib` is specified [..]")
         .with_stderr_contains("[..] `foo v0.1.0[..]` [..]")
@@ -232,7 +230,7 @@ fn doc_multiple_targets_same_name_lib() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_multiple_targets_same_name() {
     let p = project()
         .file(
@@ -241,7 +239,8 @@ fn doc_multiple_targets_same_name() {
             [workspace]
             members = ["foo", "bar"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -251,7 +250,8 @@ fn doc_multiple_targets_same_name() {
             name = "foo_lib"
             path = "src/foo_lib.rs"
         "#,
-        ).file("foo/src/foo_lib.rs", "")
+        )
+        .file("foo/src/foo_lib.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -261,10 +261,11 @@ fn doc_multiple_targets_same_name() {
             [lib]
             name = "foo_lib"
         "#,
-        ).file("bar/src/lib.rs", "")
+        )
+        .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_stderr_contains("[DOCUMENTING] foo v0.1.0 ([CWD]/foo)")
         .with_stderr_contains("[DOCUMENTING] bar v0.1.0 ([CWD]/bar)")
         .with_stderr_contains("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]")
@@ -274,7 +275,7 @@ fn doc_multiple_targets_same_name() {
     assert!(doc_file.is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_multiple_targets_same_name_bin() {
     let p = project()
         .file(
@@ -283,7 +284,8 @@ fn doc_multiple_targets_same_name_bin() {
             [workspace]
             members = ["foo", "bar"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -292,7 +294,8 @@ fn doc_multiple_targets_same_name_bin() {
             [[bin]]
             name = "foo-cli"
         "#,
-        ).file("foo/src/foo-cli.rs", "")
+        )
+        .file("foo/src/foo-cli.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -302,10 +305,11 @@ fn doc_multiple_targets_same_name_bin() {
             [[bin]]
             name = "foo-cli"
         "#,
-        ).file("bar/src/foo-cli.rs", "")
+        )
+        .file("bar/src/foo-cli.rs", "")
         .build();
 
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_status(101)
         .with_stderr_contains("[..] binary `foo_cli` is specified [..]")
         .with_stderr_contains("[..] `foo v0.1.0[..]` [..]")
@@ -313,7 +317,7 @@ fn doc_multiple_targets_same_name_bin() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_multiple_targets_same_name_undoced() {
     let p = project()
         .file(
@@ -322,7 +326,8 @@ fn doc_multiple_targets_same_name_undoced() {
             [workspace]
             members = ["foo", "bar"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -331,7 +336,8 @@ fn doc_multiple_targets_same_name_undoced() {
             [[bin]]
             name = "foo-cli"
         "#,
-        ).file("foo/src/foo-cli.rs", "")
+        )
+        .file("foo/src/foo-cli.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -342,13 +348,14 @@ fn doc_multiple_targets_same_name_undoced() {
             name = "foo-cli"
             doc = false
         "#,
-        ).file("bar/src/foo-cli.rs", "")
+        )
+        .file("bar/src/foo-cli.rs", "")
         .build();
 
-    p.cargo("doc --all").run();
+    p.cargo("doc --workspace").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_lib_bin_same_name_documents_lib() {
     let p = project()
         .file(
@@ -360,13 +367,15 @@ fn doc_lib_bin_same_name_documents_lib() {
                 foo::foo();
             }
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             r#"
             //! Library documentation
             pub fn foo() {}
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc")
         .with_stderr(
@@ -374,7 +383,8 @@ fn doc_lib_bin_same_name_documents_lib() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
     assert!(p.root().join("target/doc").is_dir());
     let doc_file = p.root().join("target/doc/foo/index.html");
     assert!(doc_file.is_file());
@@ -387,7 +397,7 @@ fn doc_lib_bin_same_name_documents_lib() {
     assert!(!doc_html.contains("Binary"));
 }
 
-#[test]
+#[cargo_test]
 fn doc_lib_bin_same_name_documents_lib_when_requested() {
     let p = project()
         .file(
@@ -399,13 +409,15 @@ fn doc_lib_bin_same_name_documents_lib_when_requested() {
                 foo::foo();
             }
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             r#"
             //! Library documentation
             pub fn foo() {}
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc --lib")
         .with_stderr(
@@ -413,7 +425,8 @@ fn doc_lib_bin_same_name_documents_lib_when_requested() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
     assert!(p.root().join("target/doc").is_dir());
     let doc_file = p.root().join("target/doc/foo/index.html");
     assert!(doc_file.is_file());
@@ -426,7 +439,7 @@ fn doc_lib_bin_same_name_documents_lib_when_requested() {
     assert!(!doc_html.contains("Binary"));
 }
 
-#[test]
+#[cargo_test]
 fn doc_lib_bin_same_name_documents_named_bin_when_requested() {
     let p = project()
         .file(
@@ -438,13 +451,15 @@ fn doc_lib_bin_same_name_documents_named_bin_when_requested() {
                 foo::foo();
             }
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             r#"
             //! Library documentation
             pub fn foo() {}
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc --bin foo")
         .with_stderr(
@@ -453,7 +468,8 @@ fn doc_lib_bin_same_name_documents_named_bin_when_requested() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
     assert!(p.root().join("target/doc").is_dir());
     let doc_file = p.root().join("target/doc/foo/index.html");
     assert!(doc_file.is_file());
@@ -466,7 +482,7 @@ fn doc_lib_bin_same_name_documents_named_bin_when_requested() {
     assert!(doc_html.contains("Binary"));
 }
 
-#[test]
+#[cargo_test]
 fn doc_lib_bin_same_name_documents_bins_when_requested() {
     let p = project()
         .file(
@@ -478,13 +494,15 @@ fn doc_lib_bin_same_name_documents_bins_when_requested() {
                 foo::foo();
             }
         "#,
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             r#"
             //! Library documentation
             pub fn foo() {}
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc --bins")
         .with_stderr(
@@ -493,7 +511,8 @@ fn doc_lib_bin_same_name_documents_bins_when_requested() {
 [DOCUMENTING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
     assert!(p.root().join("target/doc").is_dir());
     let doc_file = p.root().join("target/doc/foo/index.html");
     assert!(doc_file.is_file());
@@ -506,7 +525,7 @@ fn doc_lib_bin_same_name_documents_bins_when_requested() {
     assert!(doc_html.contains("Binary"));
 }
 
-#[test]
+#[cargo_test]
 fn doc_dash_p() {
     let p = project()
         .file(
@@ -520,7 +539,8 @@ fn doc_dash_p() {
             [dependencies.a]
             path = "a"
         "#,
-        ).file("src/lib.rs", "extern crate a;")
+        )
+        .file("src/lib.rs", "extern crate a;")
         .file(
             "a/Cargo.toml",
             r#"
@@ -532,7 +552,8 @@ fn doc_dash_p() {
             [dependencies.b]
             path = "../b"
         "#,
-        ).file("a/src/lib.rs", "extern crate b;")
+        )
+        .file("a/src/lib.rs", "extern crate b;")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "")
         .build();
@@ -545,10 +566,11 @@ fn doc_dash_p() {
 [DOCUMENTING] a v0.0.1 ([CWD]/a)
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_same_name() {
     let p = project()
         .file("src/lib.rs", "")
@@ -560,8 +582,12 @@ fn doc_same_name() {
     p.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_target() {
+    if !is_nightly() {
+        // no_core, lang_items requires nightly.
+        return;
+    }
     const TARGET: &str = "arm-unknown-linux-gnueabihf";
 
     let p = project()
@@ -578,18 +604,18 @@ fn doc_target() {
                 pub static A: u32;
             }
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc --verbose --target").arg(TARGET).run();
     assert!(p.root().join(&format!("target/{}/doc", TARGET)).is_dir());
-    assert!(
-        p.root()
-            .join(&format!("target/{}/doc/foo/index.html", TARGET))
-            .is_file()
-    );
+    assert!(p
+        .root()
+        .join(&format!("target/{}/doc/foo/index.html", TARGET))
+        .is_file());
 }
 
-#[test]
+#[cargo_test]
 fn target_specific_not_documented() {
     let p = project()
         .file(
@@ -603,7 +629,8 @@ fn target_specific_not_documented() {
             [target.foo.dependencies]
             a = { path = "a" }
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
         .file("a/src/lib.rs", "not rust")
         .build();
@@ -611,7 +638,7 @@ fn target_specific_not_documented() {
     p.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn output_not_captured() {
     let p = project()
         .file(
@@ -625,7 +652,8 @@ fn output_not_captured() {
             [dependencies]
             a = { path = "a" }
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
         .file(
             "a/src/lib.rs",
@@ -635,16 +663,17 @@ fn output_not_captured() {
             /// ```
             pub fn foo() {}
         ",
-        ).build();
+        )
+        .build();
 
     p.cargo("doc")
         .without_status()
-        .with_stderr_contains("1 | ☃")
-        .with_stderr_contains(r"error: unknown start of token: \u{2603}")
+        .with_stderr_contains("[..]☃")
+        .with_stderr_contains(r"[..]unknown start of token: \u{2603}")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn target_specific_documented() {
     let p = project()
         .file(
@@ -663,7 +692,8 @@ fn target_specific_documented() {
         "#,
                 rustc_host()
             ),
-        ).file(
+        )
+        .file(
             "src/lib.rs",
             "
             extern crate a;
@@ -671,19 +701,21 @@ fn target_specific_documented() {
             /// test
             pub fn foo() {}
         ",
-        ).file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
+        )
+        .file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
         .file(
             "a/src/lib.rs",
             "
             /// test
             pub fn foo() {}
         ",
-        ).build();
+        )
+        .build();
 
     p.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn no_document_build_deps() {
     let p = project()
         .file(
@@ -697,7 +729,8 @@ fn no_document_build_deps() {
             [build-dependencies]
             a = { path = "a" }
         "#,
-        ).file("src/lib.rs", "pub fn foo() {}")
+        )
+        .file("src/lib.rs", "pub fn foo() {}")
         .file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
         .file(
             "a/src/lib.rs",
@@ -707,12 +740,13 @@ fn no_document_build_deps() {
             /// ```
             pub fn foo() {}
         ",
-        ).build();
+        )
+        .build();
 
     p.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_release() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -724,10 +758,11 @@ fn doc_release() {
 [RUNNING] `rustdoc [..] src/lib.rs [..]`
 [FINISHED] release [optimized] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_multiple_deps() {
     let p = project()
         .file(
@@ -744,7 +779,8 @@ fn doc_multiple_deps() {
             [dependencies.baz]
             path = "baz"
         "#,
-        ).file("src/lib.rs", "extern crate bar; pub fn foo() {}")
+        )
+        .file("src/lib.rs", "extern crate bar; pub fn foo() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .file("baz/Cargo.toml", &basic_manifest("baz", "0.0.1"))
@@ -758,7 +794,7 @@ fn doc_multiple_deps() {
     assert!(p.root().join("target/doc/baz/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn features() {
     let p = project()
         .file(
@@ -775,7 +811,8 @@ fn features() {
             [features]
             foo = ["bar/bar"]
         "#,
-        ).file("src/lib.rs", r#"#[cfg(feature = "foo")] pub fn foo() {}"#)
+        )
+        .file("src/lib.rs", r#"#[cfg(feature = "foo")] pub fn foo() {}"#)
         .file(
             "bar/Cargo.toml",
             r#"
@@ -787,24 +824,27 @@ fn features() {
             [features]
             bar = []
         "#,
-        ).file(
+        )
+        .file(
             "bar/build.rs",
             r#"
             fn main() {
                 println!("cargo:rustc-cfg=bar");
             }
         "#,
-        ).file(
+        )
+        .file(
             "bar/src/lib.rs",
             r#"#[cfg(feature = "bar")] pub fn bar() {}"#,
-        ).build();
+        )
+        .build();
     p.cargo("doc --features foo").run();
     assert!(p.root().join("target/doc").is_dir());
     assert!(p.root().join("target/doc/foo/fn.foo.html").is_file());
     assert!(p.root().join("target/doc/bar/fn.bar.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn rerun_when_dir_removed() {
     let p = project()
         .file(
@@ -813,7 +853,8 @@ fn rerun_when_dir_removed() {
             /// dox
             pub fn foo() {}
         "#,
-        ).build();
+        )
+        .build();
 
     p.cargo("doc").run();
     assert!(p.root().join("target/doc/foo/index.html").is_file());
@@ -824,7 +865,7 @@ fn rerun_when_dir_removed() {
     assert!(p.root().join("target/doc/foo/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn document_only_lib() {
     let p = project()
         .file(
@@ -833,7 +874,8 @@ fn document_only_lib() {
             /// dox
             pub fn foo() {}
         "#,
-        ).file(
+        )
+        .file(
             "src/bin/bar.rs",
             r#"
             /// ```
@@ -842,16 +884,14 @@ fn document_only_lib() {
             pub fn foo() {}
             fn main() { foo(); }
         "#,
-        ).build();
+        )
+        .build();
     p.cargo("doc --lib").run();
     assert!(p.root().join("target/doc/foo/index.html").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn plugins_no_use_target() {
-    if !support::is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -864,12 +904,13 @@ fn plugins_no_use_target() {
             [lib]
             proc-macro = true
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
     p.cargo("doc --target=x86_64-unknown-openbsd -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_all_workspace() {
     let p = project()
         .file(
@@ -884,20 +925,21 @@ fn doc_all_workspace() {
 
             [workspace]
         "#,
-        ).file("src/main.rs", "fn main() {}")
+        )
+        .file("src/main.rs", "fn main() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
     // The order in which bar is compiled or documented is not deterministic
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_stderr_contains("[..] Documenting bar v0.1.0 ([..])")
         .with_stderr_contains("[..] Checking bar v0.1.0 ([..])")
         .with_stderr_contains("[..] Documenting foo v0.1.0 ([..])")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_all_virtual_manifest() {
     let p = project()
         .file(
@@ -906,20 +948,21 @@ fn doc_all_virtual_manifest() {
             [workspace]
             members = ["bar", "baz"]
         "#,
-        ).file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
     // The order in which bar and baz are documented is not guaranteed
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_stderr_contains("[..] Documenting baz v0.1.0 ([..])")
         .with_stderr_contains("[..] Documenting bar v0.1.0 ([..])")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_virtual_manifest_all_implied() {
     let p = project()
         .file(
@@ -928,7 +971,8 @@ fn doc_virtual_manifest_all_implied() {
             [workspace]
             members = ["bar", "baz"]
         "#,
-        ).file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
         .file("baz/src/lib.rs", "pub fn baz() {}")
@@ -941,12 +985,8 @@ fn doc_virtual_manifest_all_implied() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_all_member_dependency_same_name() {
-    if !is_nightly() {
-        // This can be removed once 1.29 is stable (rustdoc --cap-lints).
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -954,7 +994,8 @@ fn doc_all_member_dependency_same_name() {
             [workspace]
             members = ["bar"]
         "#,
-        ).file(
+        )
+        .file(
             "bar/Cargo.toml",
             r#"
             [project]
@@ -964,18 +1005,20 @@ fn doc_all_member_dependency_same_name() {
             [dependencies]
             bar = "0.1.0"
         "#,
-        ).file("bar/src/lib.rs", "pub fn bar() {}")
+        )
+        .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
     Package::new("bar", "0.1.0").publish();
 
-    p.cargo("doc --all")
+    p.cargo("doc --workspace")
         .with_stderr_contains("[..] Updating `[..]` index")
         .with_stderr_contains("[..] Documenting bar v0.1.0 ([..])")
         .run();
 }
 
-#[test]
+#[cargo_test]
+#[cfg(not(any(target_os = "windows", target_os = "macos")))]
 fn doc_workspace_open_help_message() {
     let p = project()
         .file(
@@ -984,29 +1027,23 @@ fn doc_workspace_open_help_message() {
             [workspace]
             members = ["foo", "bar"]
         "#,
-        ).file("foo/Cargo.toml", &basic_manifest("foo", "0.1.0"))
+        )
+        .file("foo/Cargo.toml", &basic_manifest("foo", "0.1.0"))
         .file("foo/src/lib.rs", "")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("bar/src/lib.rs", "")
         .build();
 
     // The order in which bar is compiled or documented is not deterministic
-    p.cargo("doc --all --open")
-        .with_status(101)
+    p.cargo("doc --workspace --open")
+        .env("BROWSER", "echo")
         .with_stderr_contains("[..] Documenting bar v0.1.0 ([..])")
         .with_stderr_contains("[..] Documenting foo v0.1.0 ([..])")
-        .with_stderr_contains(
-            "error: Passing multiple packages and `open` \
-             is not supported.",
-        ).with_stderr_contains(
-            "Please re-run this command with `-p <spec>` \
-             where `<spec>` is one of the following:",
-        ).with_stderr_contains("  foo")
-        .with_stderr_contains("  bar")
+        .with_stderr_contains("[..] Opening [..]/foo/index.html")
         .run();
 }
 
-#[test]
+#[cargo_test]
 #[cfg(not(any(target_os = "windows", target_os = "macos")))]
 fn doc_workspace_open_different_library_and_package_names() {
     let p = project()
@@ -1016,7 +1053,8 @@ fn doc_workspace_open_different_library_and_package_names() {
             [workspace]
             members = ["foo"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -1025,7 +1063,8 @@ fn doc_workspace_open_different_library_and_package_names() {
             [lib]
             name = "foolib"
         "#,
-        ).file("foo/src/lib.rs", "")
+        )
+        .file("foo/src/lib.rs", "")
         .build();
 
     p.cargo("doc --open")
@@ -1035,7 +1074,7 @@ fn doc_workspace_open_different_library_and_package_names() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 #[cfg(not(any(target_os = "windows", target_os = "macos")))]
 fn doc_workspace_open_binary() {
     let p = project()
@@ -1045,7 +1084,8 @@ fn doc_workspace_open_binary() {
             [workspace]
             members = ["foo"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -1055,7 +1095,8 @@ fn doc_workspace_open_binary() {
             name = "foobin"
             path = "src/main.rs"
         "#,
-        ).file("foo/src/main.rs", "")
+        )
+        .file("foo/src/main.rs", "")
         .build();
 
     p.cargo("doc --open")
@@ -1065,7 +1106,7 @@ fn doc_workspace_open_binary() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 #[cfg(not(any(target_os = "windows", target_os = "macos")))]
 fn doc_workspace_open_binary_and_library() {
     let p = project()
@@ -1075,7 +1116,8 @@ fn doc_workspace_open_binary_and_library() {
             [workspace]
             members = ["foo"]
         "#,
-        ).file(
+        )
+        .file(
             "foo/Cargo.toml",
             r#"
             [package]
@@ -1087,7 +1129,8 @@ fn doc_workspace_open_binary_and_library() {
             name = "foobin"
             path = "src/main.rs"
         "#,
-        ).file("foo/src/lib.rs", "")
+        )
+        .file("foo/src/lib.rs", "")
         .file("foo/src/main.rs", "")
         .build();
 
@@ -1098,48 +1141,37 @@ fn doc_workspace_open_binary_and_library() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_edition() {
-    if !support::is_nightly() {
-        // Stable rustdoc won't have the edition option.  Remove this once it
-        // is stabilized.
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["edition"]
             [package]
             name = "foo"
             version = "0.0.1"
             authors = []
             edition = "2018"
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
 
     p.cargo("doc -v")
-        .masquerade_as_nightly_cargo()
         .with_stderr_contains("[RUNNING] `rustdoc [..]--edition=2018[..]")
         .run();
 
     p.cargo("test -v")
-        .masquerade_as_nightly_cargo()
         .with_stderr_contains("[RUNNING] `rustdoc [..]--edition=2018[..]")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_target_edition() {
-    if !support::is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["edition"]
             [package]
             name = "foo"
             version = "0.0.1"
@@ -1148,28 +1180,23 @@ fn doc_target_edition() {
             [lib]
             edition = "2018"
         "#,
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
 
     p.cargo("doc -v")
-        .masquerade_as_nightly_cargo()
         .with_stderr_contains("[RUNNING] `rustdoc [..]--edition=2018[..]")
         .run();
 
     p.cargo("test -v")
-        .masquerade_as_nightly_cargo()
         .with_stderr_contains("[RUNNING] `rustdoc [..]--edition=2018[..]")
         .run();
 }
 
 // Tests an issue where depending on different versions of the same crate depending on `cfg`s
 // caused `cargo doc` to fail.
-#[test]
+#[cargo_test]
 fn issue_5345() {
-    if !is_nightly() {
-        // This can be removed once 1.29 is stable (rustdoc --cap-lints).
-        return;
-    }
     let foo = project()
         .file(
             "Cargo.toml",
@@ -1185,7 +1212,8 @@ fn issue_5345() {
             [target.'cfg(not(all(windows, target_arch = "x86")))'.dependencies]
             bar = "0.2"
         "#,
-        ).file("src/lib.rs", "extern crate bar;")
+        )
+        .file("src/lib.rs", "extern crate bar;")
         .build();
     Package::new("bar", "0.1.0").publish();
     Package::new("bar", "0.2.0").publish();
@@ -1194,7 +1222,7 @@ fn issue_5345() {
     foo.cargo("doc").run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_private_items() {
     let foo = project()
         .file("src/lib.rs", "mod private { fn private_item() {} }")
@@ -1202,14 +1230,13 @@ fn doc_private_items() {
     foo.cargo("doc --document-private-items").run();
 
     assert!(foo.root().join("target/doc").is_dir());
-    assert!(
-        foo.root()
-            .join("target/doc/foo/private/index.html")
-            .is_file()
-    );
+    assert!(foo
+        .root()
+        .join("target/doc/foo/private/index.html")
+        .is_file());
 }
 
-#[test]
+#[cargo_test]
 fn doc_private_ws() {
     let p = project()
         .file(
@@ -1218,20 +1245,24 @@ fn doc_private_ws() {
             [workspace]
             members = ["a", "b"]
         "#,
-        ).file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
+        )
+        .file("a/Cargo.toml", &basic_manifest("a", "0.0.1"))
         .file("a/src/lib.rs", "fn p() {}")
         .file("b/Cargo.toml", &basic_manifest("b", "0.0.1"))
         .file("b/src/lib.rs", "fn p2() {}")
         .file("b/src/main.rs", "fn main() {}")
         .build();
-    p.cargo("doc --all --bins --lib --document-private-items -v")
+    p.cargo("doc --workspace --bins --lib --document-private-items -v")
         .with_stderr_contains(
             "[RUNNING] `rustdoc [..] a/src/lib.rs [..]--document-private-items[..]",
-        ).with_stderr_contains(
+        )
+        .with_stderr_contains(
             "[RUNNING] `rustdoc [..] b/src/lib.rs [..]--document-private-items[..]",
-        ).with_stderr_contains(
+        )
+        .with_stderr_contains(
             "[RUNNING] `rustdoc [..] b/src/main.rs [..]--document-private-items[..]",
-        ).run();
+        )
+        .run();
 }
 
 const BAD_INTRA_LINK_LIB: &str = r#"
@@ -1241,16 +1272,16 @@ const BAD_INTRA_LINK_LIB: &str = r#"
 pub fn foo() {}
 "#;
 
-#[test]
+#[cargo_test]
 fn doc_cap_lints() {
     if !is_nightly() {
-        // This can be removed once 1.29 is stable (rustdoc --cap-lints).
+        // This can be removed once intra_doc_link_resolution_failure fails on stable.
         return;
     }
     let a = git::new("a", |p| {
         p.file("Cargo.toml", &basic_lib_manifest("a"))
             .file("src/lib.rs", BAD_INTRA_LINK_LIB)
-    }).unwrap();
+    });
 
     let p = project()
         .file(
@@ -1267,7 +1298,8 @@ fn doc_cap_lints() {
         "#,
                 a.url()
             ),
-        ).file("src/lib.rs", "")
+        )
+        .file("src/lib.rs", "")
         .build();
 
     p.cargo("doc")
@@ -1279,29 +1311,31 @@ fn doc_cap_lints() {
 [DOCUMENTING] foo v0.0.1 ([..])
 [FINISHED] dev [..]
 ",
-        ).run();
+        )
+        .run();
 
     p.root().join("target").rm_rf();
 
     p.cargo("doc -vv")
         .with_stderr_contains(
             "\
-[WARNING] `[bad_link]` cannot be resolved, ignoring it...
+[WARNING] `[bad_link]` cannot be resolved[..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn doc_message_format() {
     if !is_nightly() {
-        // This can be removed once 1.30 is stable (rustdoc --error-format stabilized).
+        // This can be removed once intra_doc_link_resolution_failure fails on stable.
         return;
     }
     let p = project().file("src/lib.rs", BAD_INTRA_LINK_LIB).build();
 
     p.cargo("doc --message-format=json")
         .with_status(101)
-        .with_json(
+        .with_json_contains_unordered(
             r#"
             {
                 "message": {
@@ -1317,22 +1351,162 @@ fn doc_message_format() {
                 "target": "{...}"
             }
             "#,
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn short_message_format() {
     if !is_nightly() {
-        // This can be removed once 1.30 is stable (rustdoc --error-format stabilized).
+        // This can be removed once intra_doc_link_resolution_failure fails on stable.
         return;
     }
     let p = project().file("src/lib.rs", BAD_INTRA_LINK_LIB).build();
     p.cargo("doc --message-format=short")
         .with_status(101)
-        .with_stderr_contains(
+        .with_stderr_contains("src/lib.rs:4:6: error: `[bad_link]` cannot be resolved[..]")
+        .run();
+}
+
+#[cargo_test]
+fn doc_example() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            edition = "2018"
+
+            [[example]]
+            crate-type = ["lib"]
+            name = "ex1"
+            doc = true
+            "#,
+        )
+        .file("src/lib.rs", "pub fn f() {}")
+        .file(
+            "examples/ex1.rs",
+            r#"
+            use foo::f;
+
+            /// Example
+            pub fn x() { f(); }
+            "#,
+        )
+        .build();
+
+    p.cargo("doc").run();
+    assert!(p
+        .build_dir()
+        .join("doc")
+        .join("ex1")
+        .join("fn.x.html")
+        .exists());
+}
+
+#[cargo_test]
+fn bin_private_items() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#,
+        )
+        .file(
+            "src/main.rs",
+            "
+            pub fn foo_pub() {}
+            fn foo_priv() {}
+            struct FooStruct;
+            enum FooEnum {}
+            trait FooTrait {}
+            type FooType = u32;
+            mod foo_mod {}
+
+        ",
+        )
+        .build();
+
+    p.cargo("doc")
+        .with_stderr(
             "\
-src/lib.rs:4:6: error: `[bad_link]` cannot be resolved, ignoring it...
-error: Could not document `foo`.
+[DOCUMENTING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
+
+    assert!(p.root().join("target/doc/foo/index.html").is_file());
+    assert!(p.root().join("target/doc/foo/fn.foo_pub.html").is_file());
+    assert!(p.root().join("target/doc/foo/fn.foo_priv.html").is_file());
+    assert!(p
+        .root()
+        .join("target/doc/foo/struct.FooStruct.html")
+        .is_file());
+    assert!(p.root().join("target/doc/foo/enum.FooEnum.html").is_file());
+    assert!(p
+        .root()
+        .join("target/doc/foo/trait.FooTrait.html")
+        .is_file());
+    assert!(p.root().join("target/doc/foo/type.FooType.html").is_file());
+    assert!(p.root().join("target/doc/foo/foo_mod/index.html").is_file());
+}
+
+#[cargo_test]
+fn bin_private_items_deps() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies.bar]
+            path = "bar"
+        "#,
+        )
+        .file(
+            "src/main.rs",
+            "
+            fn foo_priv() {}
+            pub fn foo_pub() {}
+        ",
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
+        .file(
+            "bar/src/lib.rs",
+            "
+            #[allow(dead_code)]
+            fn bar_priv() {}
+            pub fn bar_pub() {}
+        ",
+        )
+        .build();
+
+    p.cargo("doc")
+        .with_stderr_unordered(
+            "\
+[DOCUMENTING] bar v0.0.1 ([..])
+[CHECKING] bar v0.0.1 ([..])
+[DOCUMENTING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+",
+        )
+        .run();
+
+    assert!(p.root().join("target/doc/foo/index.html").is_file());
+    assert!(p.root().join("target/doc/foo/fn.foo_pub.html").is_file());
+    assert!(p.root().join("target/doc/foo/fn.foo_priv.html").is_file());
+
+    assert!(p.root().join("target/doc/bar/index.html").is_file());
+    assert!(p.root().join("target/doc/bar/fn.bar_pub.html").is_file());
+    assert!(!p.root().join("target/doc/bar/fn.bar_priv.html").exists());
 }
