@@ -1,14 +1,17 @@
+//! Tests for normal registry dependencies.
+
 use std::fs::{self, File};
 use std::io::prelude::*;
+use std::path::Path;
 
-use crate::support::cargo_process;
-use crate::support::git;
-use crate::support::paths::{self, CargoPathExt};
-use crate::support::registry::{self, registry_path, registry_url, Dependency, Package};
-use crate::support::{basic_manifest, project};
 use cargo::util::paths::remove_dir_all;
+use cargo_test_support::cargo_process;
+use cargo_test_support::git;
+use cargo_test_support::paths::{self, CargoPathExt};
+use cargo_test_support::registry::{self, registry_path, registry_url, Dependency, Package};
+use cargo_test_support::{basic_manifest, project, t};
 
-#[test]
+#[cargo_test]
 fn simple() {
     let p = project()
         .file(
@@ -56,7 +59,7 @@ fn simple() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn deps() {
     let p = project()
         .file(
@@ -94,7 +97,7 @@ fn deps() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn nonexistent() {
     Package::new("init", "0.0.1").publish();
 
@@ -127,7 +130,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn wrong_case() {
     Package::new("init", "0.0.1").publish();
 
@@ -162,7 +165,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn mis_hyphenated() {
     Package::new("mis-hyphenated", "0.0.1").publish();
 
@@ -197,7 +200,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn wrong_version() {
     let p = project()
         .file(
@@ -246,7 +249,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_cksum() {
     let p = project()
         .file(
@@ -284,7 +287,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_registry() {
     Package::new("init", "0.0.1").publish();
 
@@ -332,7 +335,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn package_with_path_deps() {
     Package::new("init", "0.0.1").publish();
 
@@ -362,7 +365,7 @@ fn package_with_path_deps() {
         .with_status(101)
         .with_stderr_contains(
             "\
-[ERROR] failed to verify package tarball
+[ERROR] failed to prepare local package for uploading
 
 Caused by:
   no matching package named `notyet` found
@@ -378,8 +381,8 @@ required by package `foo v0.0.1 ([..])`
         .with_stderr(
             "\
 [PACKAGING] foo v0.0.1 ([CWD])
-[VERIFYING] foo v0.0.1 ([CWD])
 [UPDATING] `[..]` index
+[VERIFYING] foo v0.0.1 ([CWD])
 [DOWNLOADING] crates ...
 [DOWNLOADED] notyet v0.0.1 (registry `[ROOT][..]`)
 [COMPILING] notyet v0.0.1
@@ -390,7 +393,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn lockfile_locks() {
     let p = project()
         .file(
@@ -429,7 +432,7 @@ fn lockfile_locks() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn lockfile_locks_transitively() {
     let p = project()
         .file(
@@ -472,7 +475,7 @@ fn lockfile_locks_transitively() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_are_not_used() {
     let p = project()
         .file(
@@ -514,7 +517,7 @@ fn yanks_are_not_used() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn relying_on_a_yank_is_bad() {
     let p = project()
         .file(
@@ -550,7 +553,7 @@ required by package `bar v0.0.1`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok() {
     let p = project()
         .file(
@@ -590,7 +593,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok_for_other_update() {
     let p = project()
         .file(
@@ -644,7 +647,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok_with_new_dep() {
     let p = project()
         .file(
@@ -687,7 +690,7 @@ fn yanks_in_lockfiles_are_ok_with_new_dep() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_with_lockfile_if_packages_missing() {
     let p = project()
         .file(
@@ -722,7 +725,7 @@ fn update_with_lockfile_if_packages_missing() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_lockfile() {
     let p = project()
         .file(
@@ -819,31 +822,7 @@ fn update_lockfile() {
         .run();
 }
 
-#[test]
-fn update_offline() {
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [dependencies]
-            bar = "*"
-        "#,
-        )
-        .file("src/main.rs", "fn main() {}")
-        .build();
-    p.cargo("update -Zoffline")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr("error: you can't update in the offline mode[..]")
-        .run();
-}
-
-#[test]
+#[cargo_test]
 fn dev_dependency_not_used() {
     let p = project()
         .file(
@@ -878,7 +857,7 @@ fn dev_dependency_not_used() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn login_with_no_cargo_dir() {
     // Create a config in the root directory because `login` requires the
     // index to be updated, and we don't want to hit crates.io.
@@ -890,7 +869,7 @@ fn login_with_no_cargo_dir() {
     assert_eq!(credentials, "[registry]\ntoken = \"foo\"\n");
 }
 
-#[test]
+#[cargo_test]
 fn login_with_differently_sized_token() {
     // Verify that the configuration file gets properly truncated.
     registry::init();
@@ -903,7 +882,7 @@ fn login_with_differently_sized_token() {
     assert_eq!(credentials, "[registry]\ntoken = \"lmaolmaolmao\"\n");
 }
 
-#[test]
+#[cargo_test]
 fn bad_license_file() {
     Package::new("foo", "1.0.0").publish();
     let p = project()
@@ -928,7 +907,7 @@ fn bad_license_file() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn updating_a_dep() {
     let p = project()
         .file(
@@ -1004,7 +983,7 @@ fn updating_a_dep() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn git_and_registry_dep() {
     let b = git::repo(&paths::root().join("b"))
         .file(
@@ -1066,7 +1045,7 @@ fn git_and_registry_dep() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_publish_then_update() {
     // First generate a Cargo.lock and a clone of the registry index at the
     // "head" of the current registry.
@@ -1138,7 +1117,7 @@ fn update_publish_then_update() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fetch_downloads() {
     let p = project()
         .file(
@@ -1169,7 +1148,7 @@ fn fetch_downloads() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_transitive_dependency() {
     let p = project()
         .file(
@@ -1217,7 +1196,7 @@ fn update_transitive_dependency() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_backtracking_ok() {
     let p = project()
         .file(
@@ -1266,7 +1245,7 @@ fn update_backtracking_ok() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_multiple_packages() {
     let p = project()
         .file(
@@ -1326,7 +1305,7 @@ fn update_multiple_packages() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bundled_crate_in_registry() {
     let p = project()
         .file(
@@ -1368,7 +1347,7 @@ fn bundled_crate_in_registry() {
     p.cargo("run").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_same_prefix_oh_my_how_was_this_a_bug() {
     let p = project()
         .file(
@@ -1395,7 +1374,7 @@ fn update_same_prefix_oh_my_how_was_this_a_bug() {
     p.cargo("update -pfoobar --precise=0.2.0").run();
 }
 
-#[test]
+#[cargo_test]
 fn use_semver() {
     let p = project()
         .file(
@@ -1418,7 +1397,56 @@ fn use_semver() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
+fn use_semver_package_incorrectly() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["a", "b"]
+            "#,
+        )
+        .file(
+            "a/Cargo.toml",
+            r#"
+            [project]
+            name = "a"
+            version = "0.1.1-alpha.0"
+            authors = []
+            "#,
+        )
+        .file(
+            "b/Cargo.toml",
+            r#"
+            [project]
+            name = "b"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            a = { version = "^0.1", path = "../a" }
+            "#,
+        )
+        .file("a/src/main.rs", "fn main() {}")
+        .file("b/src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
+            "\
+error: no matching package named `a` found
+location searched: [..]
+prerelease package needs to be specified explicitly
+a = { version = \"0.1.1-alpha.0\" }
+required by package `b v0.1.0 ([..])`
+",
+        )
+        .run();
+}
+
+#[cargo_test]
 fn only_download_relevant() {
     let p = project()
         .file(
@@ -1458,7 +1486,7 @@ fn only_download_relevant() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn resolve_and_backtracking() {
     let p = project()
         .file(
@@ -1484,7 +1512,7 @@ fn resolve_and_backtracking() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn upstream_warnings_on_extra_verbose() {
     let p = project()
         .file(
@@ -1511,7 +1539,7 @@ fn upstream_warnings_on_extra_verbose() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn disallow_network() {
     let p = project()
         .file(
@@ -1545,7 +1573,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn add_dep_dont_update_registry() {
     let p = project()
         .file(
@@ -1603,7 +1631,7 @@ fn add_dep_dont_update_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bump_version_dont_update_registry() {
     let p = project()
         .file(
@@ -1660,7 +1688,7 @@ fn bump_version_dont_update_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn old_version_req() {
     let p = project()
         .file(
@@ -1714,7 +1742,7 @@ this warning.
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn old_version_req_upstream() {
     let p = project()
         .file(
@@ -1773,7 +1801,7 @@ this warning.
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn toml_lies_but_index_is_truth() {
     Package::new("foo", "0.2.0").publish();
     Package::new("bar", "0.3.0")
@@ -1812,7 +1840,7 @@ fn toml_lies_but_index_is_truth() {
     p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn vv_prints_warnings() {
     Package::new("foo", "0.2.0")
         .file(
@@ -1840,7 +1868,7 @@ fn vv_prints_warnings() {
     p.cargo("build -vv").run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_and_or_malicious_packages_rejected() {
     Package::new("foo", "0.2.0")
         .extra_file("foo-0.1.0/src/lib.rs", "")
@@ -1881,7 +1909,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn git_init_templatedir_missing() {
     Package::new("foo", "0.2.0").dep("bar", "*").publish();
     Package::new("bar", "0.2.0").publish();
@@ -1919,7 +1947,7 @@ fn git_init_templatedir_missing() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn rename_deps_and_features() {
     Package::new("foo", "0.1.0")
         .file("src/lib.rs", "pub fn f1() {}")
@@ -1977,3 +2005,76 @@ fn rename_deps_and_features() {
     p.cargo("build --features bar/foo01").run();
     p.cargo("build --features bar/another").run();
 }
+
+#[cargo_test]
+fn ignore_invalid_json_lines() {
+    Package::new("foo", "0.1.0").publish();
+    Package::new("foo", "0.1.1").invalid_json(true).publish();
+    Package::new("foo", "0.2.0").publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [project]
+                name = "a"
+                version = "0.5.0"
+                authors = []
+
+                [dependencies]
+                foo = '0.1.0'
+                foo02 = { version = '0.2.0', package = 'foo' }
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("build").run();
+}
+
+#[cargo_test]
+fn readonly_registry_still_works() {
+    Package::new("foo", "0.1.0").publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [project]
+                name = "a"
+                version = "0.5.0"
+                authors = []
+
+                [dependencies]
+                foo = '0.1.0'
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("generate-lockfile").run();
+    p.cargo("fetch --locked").run();
+    chmod_readonly(&paths::home(), true);
+    p.cargo("build").run();
+    // make sure we un-readonly the files afterwards so "cargo clean" can remove them (#6934)
+    chmod_readonly(&paths::home(), false);
+
+    fn chmod_readonly(path: &Path, readonly: bool) {
+        for entry in t!(path.read_dir()) {
+            let entry = t!(entry);
+            let path = entry.path();
+            if t!(entry.file_type()).is_dir() {
+                chmod_readonly(&path, readonly);
+            } else {
+                set_readonly(&path, readonly);
+            }
+        }
+        set_readonly(path, readonly);
+    }
+
+    fn set_readonly(path: &Path, readonly: bool) {
+        let mut perms = t!(path.metadata()).permissions();
+        perms.set_readonly(readonly);
+        t!(fs::set_permissions(path, perms));
+    }
+}
